@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2018-2021 Rerrah
+ * Copyright (C) 2018-2022 Rerrah
  *
  * Permission is hereby granted, free of charge, to any person
  * obtaining a copy of this software and associated documentation
@@ -27,6 +27,7 @@
 #include <algorithm>
 #include <utility>
 #include <exception>
+#include <iterator>
 #include <unordered_map>
 #include <unordered_set>
 #include "configuration.hpp"
@@ -80,6 +81,8 @@ BambooTracker::BambooTracker(std::weak_ptr<Configuration> config)
 
 	storeOnlyUsedSamples_ = config.lock()->getWriteOnlyUsedSamples();
 	volFMReversed_ = config.lock()->getReverseFMVolumeOrder();
+
+	makeNewModule();
 }
 
 BambooTracker::~BambooTracker() = default;
@@ -468,6 +471,63 @@ std::multiset<int> BambooTracker::getPitchFMUsers(int ptNum) const
 	return instMan_->getPitchFMUsers(ptNum);
 }
 
+void BambooTracker::addPanFMSequenceData(int panNum, int data)
+{
+	instMan_->addPanFMSequenceData(panNum, data);
+}
+
+void BambooTracker::removePanFMSequenceData(int panNum)
+{
+	instMan_->removePanFMSequenceData(panNum);
+}
+
+void BambooTracker::setPanFMSequenceData(int panNum, int cnt, int data)
+{
+	instMan_->setPanFMSequenceData(panNum, cnt, data);
+}
+
+void BambooTracker::addPanFMLoop(int panNum, const InstrumentSequenceLoop& loop)
+{
+	instMan_->addPanFMLoop(panNum, loop);
+}
+
+void BambooTracker::removePanFMLoop(int panNum, int begin, int end)
+{
+	instMan_->removePanFMLoop(panNum, begin, end);
+}
+
+void BambooTracker::changePanFMLoop(int panNum, int prevBegin, int prevEnd, const InstrumentSequenceLoop& loop)
+{
+	instMan_->changePanFMLoop(panNum, prevBegin, prevEnd, loop);
+}
+
+void BambooTracker::clearPanFMLoops(int panNum)
+{
+	instMan_->clearPanFMLoops(panNum);
+}
+
+void BambooTracker::setPanFMRelease(int panNum, const InstrumentSequenceRelease& release)
+{
+	instMan_->setPanFMRelease(panNum, release);
+}
+
+void BambooTracker::setInstrumentFMPan(int instNum, int panNum)
+{
+	instMan_->setInstrumentFMPan(instNum, panNum);
+	opnaCtrl_->updateInstrumentFM(instNum);
+}
+
+void BambooTracker::setInstrumentFMPanEnabled(int instNum, bool enabled)
+{
+	instMan_->setInstrumentFMPanEnabled(instNum, enabled);
+	opnaCtrl_->updateInstrumentFM(instNum);
+}
+
+std::multiset<int> BambooTracker::getPanFMUsers(int panNum) const
+{
+	return instMan_->getPanFMUsers(panNum);
+}
+
 void BambooTracker::setInstrumentFMEnvelopeResetEnabled(int instNum, FMOperatorType op, bool enabled)
 {
 	instMan_->setInstrumentFMEnvelopeResetEnabled(instNum, op, enabled);
@@ -1056,6 +1116,63 @@ std::multiset<int> BambooTracker::getPitchADPCMUsers(int ptNum) const
 	return instMan_->getPitchADPCMUsers(ptNum);
 }
 
+void BambooTracker::addPanADPCMSequenceData(int panNum, int data)
+{
+	instMan_->addPanADPCMSequenceData(panNum, data);
+}
+
+void BambooTracker::removePanADPCMSequenceData(int panNum)
+{
+	instMan_->removePanADPCMSequenceData(panNum);
+}
+
+void BambooTracker::setPanADPCMSequenceData(int panNum, int cnt, int data)
+{
+	instMan_->setPanADPCMSequenceData(panNum, cnt, data);
+}
+
+void BambooTracker::addPanADPCMLoop(int panNum, const InstrumentSequenceLoop& loop)
+{
+	instMan_->addPanADPCMLoop(panNum, loop);
+}
+
+void BambooTracker::removePanADPCMLoop(int panNum, int begin, int end)
+{
+	instMan_->removePanADPCMLoop(panNum, begin, end);
+}
+
+void BambooTracker::changePanADPCMLoop(int panNum, int prevBegin, int prevEnd, const InstrumentSequenceLoop& loop)
+{
+	instMan_->changePanADPCMLoop(panNum, prevBegin, prevEnd, loop);
+}
+
+void BambooTracker::clearPanADPCMLoops(int panNum)
+{
+	instMan_->clearPanADPCMLoops(panNum);
+}
+
+void BambooTracker::setPanADPCMRelease(int panNum, const InstrumentSequenceRelease& release)
+{
+	instMan_->setPanADPCMRelease(panNum, release);
+}
+
+void BambooTracker::setInstrumentADPCMPan(int instNum, int panNum)
+{
+	instMan_->setInstrumentADPCMPan(instNum, panNum);
+	opnaCtrl_->updateInstrumentADPCM(instNum);
+}
+
+void BambooTracker::setInstrumentADPCMPanEnabled(int instNum, bool enabled)
+{
+	instMan_->setInstrumentADPCMPanEnabled(instNum, enabled);
+	opnaCtrl_->updateInstrumentADPCM(instNum);
+}
+
+std::multiset<int> BambooTracker::getPanADPCMUsers(int panNum) const
+{
+	return instMan_->getPanADPCMUsers(panNum);
+}
+
 //--- Drumkit
 void BambooTracker::setInstrumentDrumkitSample(int instNum, int key, int sampNum)
 {
@@ -1075,6 +1192,12 @@ void BambooTracker::setInstrumentDrumkitPitch(int instNum, int key, int pitch)
 	opnaCtrl_->updateInstrumentDrumkit(instNum, key);
 }
 
+void BambooTracker::setInstrumentDrumkitPan(int instNum, int key, int pan)
+{
+	instMan_->setInstrumentDrumkitPan(instNum, key, pan);
+	opnaCtrl_->updateInstrumentDrumkit(instNum, key);
+}
+
 /********** Song edit **********/
 int BambooTracker::getCurrentSongNumber() const
 {
@@ -1381,7 +1504,7 @@ bool BambooTracker::assignADPCMBeforeForcedJamKeyOn(
 		opnaCtrl_->clearSamplesADPCM();
 		if (opnaCtrl_->storeSampleADPCM(
 					std::dynamic_pointer_cast<InstrumentADPCM>(inst)->getRawSample(), start, stop)) {
-			sampAddrs[0] = { start, stop };
+			sampAddrs[0] = {{ start, stop }};
 			isAssignedAll = true;
 		}
 		break;
@@ -1396,7 +1519,7 @@ bool BambooTracker::assignADPCMBeforeForcedJamKeyOn(
 			int n = kit->getSampleNumber(key);
 			if (!sampAddrs.count(n)) {
 				bool assigned = opnaCtrl_->storeSampleADPCM(kit->getRawSample(key), start, stop);
-				if (assigned) sampAddrs[n] = { start, stop };
+				if (assigned) sampAddrs[n] = {{ start, stop }};
 				isAssignedAll &= assigned;
 			}
 		}
@@ -1634,6 +1757,7 @@ bool BambooTracker::exportToWav(io::WavContainer& container, int loopCnt, Export
 					if (checkFunc()) {	// Update lambda function
 						stopPlaySong();
 						isFollowPlay_ = tmpFollow;
+						opnaCtrl_->setRate(tmpRate);
 						return false;
 					}
 
@@ -1709,6 +1833,7 @@ bool BambooTracker::exportToVgm(io::BinaryContainer& container, int target, bool
 			if (checkFunc()) {	// Update lambda function
 				stopPlaySong();
 				isFollowPlay_ = tmpFollow;
+				opnaCtrl_->setRate(tmpRate);
 				return false;
 			}
 
@@ -1772,6 +1897,7 @@ bool BambooTracker::exportToS98(io::BinaryContainer& container, int target, bool
 			if (checkFunc()) {	// Update lambda function
 				stopPlaySong();
 				isFollowPlay_ = tmpFollow;
+				opnaCtrl_->setRate(tmpRate);
 				return false;
 			}
 
@@ -1805,21 +1931,19 @@ bool BambooTracker::exportToS98(io::BinaryContainer& container, int target, bool
 }
 
 /********** Real chip interface **********/
-void BambooTracker::useSCCI(scci::SoundInterfaceManager* manager)
+void BambooTracker::connectToRealChip(RealChipInterfaceType type, RealChipInterfaceGeneratorFunc* f)
 {
-	opnaCtrl_->useSCCI(manager);
+	opnaCtrl_->connectToRealChip(type, f);
 }
 
-void BambooTracker::useC86CTL(C86ctlBase* base)
+RealChipInterfaceType BambooTracker::getRealChipInterfaceType() const
 {
-	opnaCtrl_->useC86CTL(base);
+	return opnaCtrl_->getRealChipInterfaceType();
 }
 
-RealChipInterface BambooTracker::getRealChipinterface() const
+bool BambooTracker::hasConnectedToRealChip() const
 {
-	if (opnaCtrl_->isUsedSCCI()) return RealChipInterface::SCCI;
-	else if (opnaCtrl_->isUsedC86CTL()) return RealChipInterface::C86CTL;
-	else return RealChipInterface::NONE;
+	return opnaCtrl_->hasConnectedToRealChip();
 }
 
 /********** Stream events **********/
@@ -1901,6 +2025,11 @@ void BambooTracker::setMasterVolumeSSG(double dB)
 /********** Module details **********/
 /*----- Module -----*/
 void BambooTracker::makeNewModule()
+{
+	makeNewModule(true);
+}
+
+void BambooTracker::makeNewModule(bool withInstrument)
 {
 	stopPlaySong();
 
@@ -1913,14 +2042,21 @@ void BambooTracker::makeNewModule()
 	tickCounter_->setInterruptRate(mod_->getTickFrequency());
 
 	setCurrentSongNumber(0);
-	curInstNum_ = -1;
+
+	if (withInstrument) {
+		addInstrument(0, InstrumentType::FM, u8"Instrument 00");
+		curInstNum_ = 0;
+	}
+	else {
+		curInstNum_ = -1;
+	}
 
 	clearCommandHistory();
 }
 
 void BambooTracker::loadModule(io::BinaryContainer& container)
 {
-	makeNewModule();
+	makeNewModule(false);
 
 	std::exception_ptr ep;
 	try {
@@ -2181,7 +2317,7 @@ void BambooTracker::swapTracks(int songNum, int track1, int track2)
 	mod_->getSong(songNum).swapTracks(track1, track2);
 }
 
-double BambooTracker::getApproximateSongLength(int songNum) const
+double BambooTracker::estimateSongLength(int songNum) const
 {
 	SongLengthCalculator calc(*mod_.get(), songNum);
 	return calc.approximateLengthBySecond();
@@ -2255,6 +2391,42 @@ size_t BambooTracker::getBookmarkSize(int songNum) const
 	return mod_->getSong(songNum).getBookmarkSize();
 }
 
+/*----- Key signature -----*/
+void BambooTracker::addKeySignature(int songNum, KeySignature::Type key, int order, int step)
+{
+	mod_->getSong(songNum).addKeySignature(key, order, step);
+}
+
+void BambooTracker::changeKeySignature(int songNum, int i, KeySignature::Type key, int order, int step)
+{
+	mod_->getSong(songNum).changeKeySignature(i, key, order, step);
+}
+
+void BambooTracker::removeKeySignature(int songNum, int i)
+{
+	mod_->getSong(songNum).removeKeySignature(i);
+}
+
+void BambooTracker::clearKeySignature(int songNum)
+{
+	mod_->getSong(songNum).clearKeySignature();
+}
+
+KeySignature BambooTracker::getKeySignature(int songNum, int i) const
+{
+	return mod_->getSong(songNum).getKeySignature(i);
+}
+
+size_t BambooTracker::getKeySignatureSize(int songNum) const
+{
+	return mod_->getSong(songNum).getKeySignatureSize();
+}
+
+KeySignature::Type BambooTracker::searchKeySignatureAt(int songNum, int order, int step) const
+{
+	return mod_->getSong(songNum).searchKeySignatureAt(order, step);
+}
+
 /*----- Track -----*/
 void BambooTracker::setEffectDisplayWidth(int songNum, int trackNum, size_t w)
 {
@@ -2266,6 +2438,16 @@ size_t BambooTracker::getEffectDisplayWidth(int songNum, int trackNum) const
 	return mod_->getSong(songNum).getTrack(trackNum).getEffectDisplayWidth();
 }
 
+void BambooTracker::setTrackVisibility(int songNum, int trackNum, bool visible)
+{
+	mod_->getSong(songNum).getTrack(trackNum).setVisibility(visible);
+}
+
+bool BambooTracker::isVisibleTrack(int songNum, int trackNum)
+{
+	return mod_->getSong(songNum).getTrack(trackNum).isVisible();
+}
+
 /*----- Order -----*/
 std::vector<OrderInfo> BambooTracker::getOrderData(int songNum, int orderNum) const
 {
