@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2018-2021 Rerrah
+ * Copyright (C) 2018-2022 Rerrah
  *
  * Permission is hereby granted, free of charge, to any person
  * obtaining a copy of this software and associated documentation
@@ -37,8 +37,7 @@
 #include "instrument.hpp"
 #include "module.hpp"
 #include "command/command_manager.hpp"
-#include "chip/c86ctl/c86ctl_wrapper.hpp"
-#include "chip/scci/scci.hpp"
+#include "chip/real_chip_interface.hpp"
 #include "io/binary_container.hpp"
 #include "io/export_io.hpp"
 #include "io/wav_container.hpp"
@@ -47,7 +46,6 @@
 
 class Configuration;
 enum class EffectDisplayControl;
-enum class RealChipInterface;
 class AbstractBank;
 class OPNAController;
 class PlaybackManager;
@@ -146,6 +144,18 @@ public:
 	void setInstrumentFMPitchEnabled(int instNum, FMOperatorType op, bool enabled);
 	std::multiset<int> getPitchFMUsers(int ptNum) const;
 
+	void addPanFMSequenceData(int panNum, int data);
+	void removePanFMSequenceData(int panNum);
+	void setPanFMSequenceData(int panNum, int cnt, int data);
+	void addPanFMLoop(int panNum, const InstrumentSequenceLoop& loop);
+	void removePanFMLoop(int panNum, int begin, int end);
+	void changePanFMLoop(int panNum, int prevBegin, int prevEnd, const InstrumentSequenceLoop& loop);
+	void clearPanFMLoops(int panNum);
+	void setPanFMRelease(int panNum, const InstrumentSequenceRelease& release);
+	void setInstrumentFMPan(int instNum, int panNum);
+	void setInstrumentFMPanEnabled(int instNum, bool enabled);
+	std::multiset<int> getPanFMUsers(int panNum) const;
+
 	void setInstrumentFMEnvelopeResetEnabled(int instNum, FMOperatorType op, bool enabled);
 
 	//--- SSG
@@ -269,10 +279,23 @@ public:
 	void setInstrumentADPCMPitchEnabled(int instNum, bool enabled);
 	std::multiset<int> getPitchADPCMUsers(int ptNum) const;
 
+	void addPanADPCMSequenceData(int panNum, int data);
+	void removePanADPCMSequenceData(int panNum);
+	void setPanADPCMSequenceData(int panNum, int cnt, int data);
+	void addPanADPCMLoop(int panNum, const InstrumentSequenceLoop& loop);
+	void removePanADPCMLoop(int panNum, int begin, int end);
+	void changePanADPCMLoop(int panNum, int prevBegin, int prevEnd, const InstrumentSequenceLoop& loop);
+	void clearPanADPCMLoops(int panNum);
+	void setPanADPCMRelease(int panNum, const InstrumentSequenceRelease& release);
+	void setInstrumentADPCMPan(int instNum, int panNum);
+	void setInstrumentADPCMPanEnabled(int instNum, bool enabled);
+	std::multiset<int> getPanADPCMUsers(int panNum) const;
+
 	//--- Drumkit
 	void setInstrumentDrumkitSample(int instNum, int key, int sampNum);
 	void setInstrumentDrumkitSampleEnabled(int instNum, int key, bool enabled);
 	void setInstrumentDrumkitPitch(int instNum, int key, int pitch);
+	void setInstrumentDrumkitPan(int instNum, int key, int pan);
 
 	// Song edit
 	void setCurrentSongNumber(int num);
@@ -333,9 +356,9 @@ public:
 					 const io::S98Tag& tag, int rate, ExportCancellCallback checkFunc);
 
 	// Real chip interface
-	void useSCCI(scci::SoundInterfaceManager* manager);
-	void useC86CTL(C86ctlBase* base);
-	RealChipInterface getRealChipinterface() const;
+	void connectToRealChip(RealChipInterfaceType type, RealChipInterfaceGeneratorFunc* f = nullptr);
+	RealChipInterfaceType getRealChipInterfaceType() const;
+	bool hasConnectedToRealChip() const;
 
 	// Stream events
 	/// 0<: Tick
@@ -409,7 +432,7 @@ public:
 	void sortSongs(const std::vector<int>& numbers);
 	void transposeSong(int songNum, int seminotes, const std::vector<int>& excludeInsts);
 	void swapTracks(int songNum, int track1, int track2);
-	double getApproximateSongLength(int songNum) const;
+	double estimateSongLength(int songNum) const;
 	size_t getTotalStepCount(int songNum, size_t loopCnt) const;
 	/*----- Bookmark -----*/
 	void addBookmark(int songNum, const std::string& name, int order, int step);
@@ -424,9 +447,19 @@ public:
 	Bookmark getPreviousBookmark(int songNum, int order, int step);
 	Bookmark getNextBookmark(int songNum, int order, int step);
 	size_t getBookmarkSize(int songNum) const;
+	/*----- Key signature -----*/
+	void addKeySignature(int songNum, KeySignature::Type key, int order, int step);
+	void changeKeySignature(int songNum, int i, KeySignature::Type key, int order, int step);
+	void removeKeySignature(int songNum, int i);
+	void clearKeySignature(int songNum);
+	KeySignature getKeySignature(int songNum, int i) const;
+	size_t getKeySignatureSize(int songNum) const;
+	KeySignature::Type searchKeySignatureAt(int songNum, int order, int step) const;
 	/*----- Track -----*/
 	void setEffectDisplayWidth(int songNum, int trackNum, size_t w);
 	size_t getEffectDisplayWidth(int songNum, int trackNum) const;
+	void setTrackVisibility(int songNum, int trackNum, bool visible);
+	bool isVisibleTrack(int songNum, int trackNum);
 	/*----- Order -----*/
 	std::vector<OrderInfo> getOrderData(int songNum, int orderNum) const;
 	void setOrderPatternDigit(int songNum, int trackNum, int orderNum, int patternNum, bool secondEntry);
@@ -464,7 +497,7 @@ public:
 	///		0: note
 	///		1: instrument
 	///		2: volume
-	///		3: effect id
+	///		3: effect ID
 	///		4: effect value
 	void pastePatternCells(int songNum, int beginTrack, int beginColmn, int beginOrder, int beginStep,
 						   const std::vector<std::vector<std::string>>& cells);
@@ -521,6 +554,9 @@ private:
 	bool isFollowPlay_;
 	bool storeOnlyUsedSamples_;
 
+	// Module details
+	void makeNewModule(bool withInstrument);
+
 	// Jam mode
 	void funcJamKeyOn(JamKey key, int keyNum, const TrackAttribute& attrib, bool volumeSet,
 					  std::shared_ptr<AbstractInstrument> inst = nullptr);
