@@ -25,6 +25,7 @@
 
 #include "configuration.hpp"
 #include "jamming.hpp"
+#include "chip/real_chip_interface.hpp"
 
 namespace
 {
@@ -160,6 +161,7 @@ Configuration::Configuration()
 	instADPCMH_ = 430;
 	instKitW_ = 590;
 	instKitH_ = 430;
+	instKitHSplit_ = -1;	// Dummy
 
 	// Toolbar state
 	mainTb_.setPosition(ToolbarPosition::TopPosition);
@@ -203,6 +205,9 @@ Configuration::Configuration()
 	// Wave view
 	waveViewFps_ = 30;
 
+	// Note names
+	notationSys_ = NoteNotationSystem::ENGLISH;
+
 	// Keys
 	shortcuts_ = {
 		{ ShortcutAction::KeyOff, u8"-" },
@@ -280,7 +285,7 @@ Configuration::Configuration()
 	// Sound //
 	sndAPI_ = u8"";
 	sndDevice_ = u8"";
-	realChip_ = RealChipInterface::NONE;
+	realChip_ = RealChipInterfaceType::NONE;
 	emulator_ = 1;
 	sampleRate_ = 44100;
 	bufferLength_ = 40;
