@@ -44,10 +44,10 @@
 #include "audio/audio_stream.hpp"
 #include "midi/midi.hpp"
 #include "jamming.hpp"
-#include "slider_style.hpp"
-#include "fm_envelope_set_edit_dialog.hpp"
-#include "color_palette_handler.hpp"
-#include "gui_utils.hpp"
+#include "gui/slider_style.hpp"
+#include "gui/fm_envelope_set_edit_dialog.hpp"
+#include "gui/note_name_manager.hpp"
+#include "gui/gui_utils.hpp"
 
 namespace
 {
@@ -60,6 +60,18 @@ inline bool fromCheckState(Qt::CheckState state)
 {
 	return (state == Qt::Checked) ? true : false;
 }
+
+struct NotationSystemAttribute
+{
+	const QString name;
+	const char* dispName;
+	const NoteNotationSystem ev;
+};
+
+const NotationSystemAttribute NOTATION_SYSS[] = {
+	{ "English", QT_TRANSLATE_NOOP("NotationSystem", "English"), NoteNotationSystem::ENGLISH },
+	{ "German", QT_TRANSLATE_NOOP("NotationSystem", "German"), NoteNotationSystem::GERMAN }
+};
 }
 
 ConfigurationDialog::ConfigurationDialog(std::weak_ptr<Configuration> config, std::weak_ptr<ColorPalette> palette,
@@ -72,7 +84,7 @@ ConfigurationDialog::ConfigurationDialog(std::weak_ptr<Configuration> config, st
 {
 	ui->setupUi(this);
 
-	setWindowFlags(windowFlags() ^ Qt::WindowContextHelpButtonHint);
+	setWindowFlags(windowFlags() & ~Qt::WindowContextHelpButtonHint);
 	QObject::connect(ui->buttonBox->button(QDialogButtonBox::Apply), &QPushButton::clicked,
 					 this, [&] {
 		on_ConfigurationDialog_accepted();
@@ -88,13 +100,13 @@ ConfigurationDialog::ConfigurationDialog(std::weak_ptr<Configuration> config, st
 		item->setData(Qt::UserRole, desc);
 	};
 	glfunc(0, configLocked->getWarpCursor(),
-		   tr("Warp the cursor around the edges of the pattern editor."));
+		   tr("Wrap the cursor around the edges of the pattern editor."));
 	glfunc(1, configLocked->getWarpAcrossOrders(),
-		   tr("Move to previous or next order when reaching top or bottom in the pattern editor."));
+		   tr("Move to the previous or next order when reaching the top or bottom in the pattern editor."));
 	glfunc(2, configLocked->getShowRowNumberInHex(),
 		   tr("Display row numbers and the playback position on the status bar in hexadecimal."));
 	glfunc(3, configLocked->getShowPreviousNextOrders(),
-		   tr("Preview previous and next orders in the pattern editor."));
+		   tr("Preview the previous and next orders in the pattern editor."));
 	glfunc(4, configLocked->getBackupModules(),
 		   tr("Create a backup copy of the existing file when saving a module."));
 	glfunc(5, configLocked->getDontSelectOnDoubleClick(),
@@ -102,7 +114,7 @@ ConfigurationDialog::ConfigurationDialog(std::weak_ptr<Configuration> config, st
 	glfunc(6, configLocked->getReverseFMVolumeOrder(),
 		   tr("Reverse the order of FM volume so that 00 is the quietest in the pattern editor."));
 	glfunc(7, configLocked->getMoveCursorToRight(),
-		   tr("Move the cursor to right after entering effects in the pattern editor."));
+		   tr("Automatically move the cursor right after entering effects in the pattern editor."));
 	glfunc(8, configLocked->getRetrieveChannelState(),
 		   tr("Reconstruct the current channel's state from previous orders upon playing."));
 	glfunc(9, configLocked->getEnableTranslation(),
@@ -110,21 +122,21 @@ ConfigurationDialog::ConfigurationDialog(std::weak_ptr<Configuration> config, st
 	glfunc(10, configLocked->getShowFMDetuneAsSigned(),
 		   tr("Display FM detune values as signed numbers in the FM envelope editor."));
 	glfunc(11, configLocked->getFill00ToEffectValue(),
-		   tr("Fill 00 to effect value column upon entering effect id."));
+		   tr("Automatically fill 00 to the effect value column upon entering an effect ID."));
 	glfunc(12, configLocked->getMoveCursorByHorizontalScroll(),
-		   tr("Move the cursor position by cell with horizontal scroll bar in the order list and the pattern editor."));
+		   tr("Move the cursor with the horizontal scroll bar in the order list and the pattern editor."));
 	glfunc(13, configLocked->getOverwriteUnusedUneditedPropety(),
-		   tr("Overwrite unused and unedited instrument properties on creating new properties. "
-			  "When disabled, override unused properties regardless of editing."));
+		   tr("Overwrite unused and unedited instrument properties when creating new properties. "
+			  "If disabled, override unused properties regardless of editing."));
 	glfunc(14, configLocked->getWriteOnlyUsedSamples(),
-		   tr("Send only ADPCM samples used by instruments to the ADPCM memory. "
-			  "Recommend to turn off if you change ADPCM samples frequently due to take the high rewriting cost."));
+		   tr("Only send samples used by instruments to the ADPCM memory. "
+			  "It is recommended to turn this off if you change ADPCM samples frequently due to the high cost of rewriting.."));
 	glfunc(15, configLocked->getReflectInstrumentNumberChange(),
-		   tr("Correspond the instrument number in patterns when the instrument changes its number."));
+		   tr("Automatically update the instrument number in patterns when an instrument's number is changed."));
 	glfunc(16, configLocked->getFixJammingVolume(),
 		   tr("Set maximum volume during jam mode. When unchecked, the volume is changed by the volume spinbox."));
 	glfunc(17, configLocked->getMuteHiddenTracks(),
-		   tr("Mute hidden tracks when visibility of tracks is changed."));
+		   tr("Automatically mute tracks when they are hidden."));
 	glfunc(18, configLocked->getRestoreTrackVisibility(),
 		   tr("Restore the previous track visibility on startup."));
 
@@ -134,6 +146,17 @@ ConfigurationDialog::ConfigurationDialog(std::weak_ptr<Configuration> config, st
 	// Wave view
 	ui->waveViewRateSpinBox->setValue(configLocked->getWaveViewFrameRate());
 
+	// Note names
+	{
+		QSignalBlocker blocker(ui->noteNameComboBox);
+		for (const auto& attrib : NOTATION_SYSS) {
+			ui->noteNameComboBox->addItem(QCoreApplication::translate("NotationSystem", attrib.dispName));
+			if (attrib.ev == configLocked->getNotationSystem()) {
+				ui->noteNameComboBox->setCurrentIndex(ui->noteNameComboBox->count() - 1);
+			}
+		}
+	}
+
 	// Keys
 	ui->shortcutsTreeWidget->header()->setSectionResizeMode(0, QHeaderView::Stretch);
 	ui->shortcutsTreeWidget->header()->setSectionResizeMode(1, QHeaderView::Fixed);
@@ -174,10 +197,10 @@ ConfigurationDialog::ConfigurationDialog(std::weak_ptr<Configuration> config, st
 		{ Configuration::ShortcutAction::ToggleBookmark, tr("Toggle bookmark") },
 		{ Configuration::ShortcutAction::PrevBookmark, tr("Previous bookmark") },
 		{ Configuration::ShortcutAction::NextBookmark, tr("Next bookmark") },
-		{ Configuration::ShortcutAction::DecreaseNote, tr("Transpose, decrease note") },
-		{ Configuration::ShortcutAction::IncreaseNote, tr("Transpose, increase note") },
-		{ Configuration::ShortcutAction::DecreaseOctave, tr("Transpose, decrease octave") },
-		{ Configuration::ShortcutAction::IncreaseOctave, tr("Transpose, increase octave") },
+		{ Configuration::ShortcutAction::DecreaseNote, tr("Transpose down one semitone") },
+		{ Configuration::ShortcutAction::IncreaseNote, tr("Transpose up one semitone") },
+		{ Configuration::ShortcutAction::DecreaseOctave, tr("Transpose down one octave") },
+		{ Configuration::ShortcutAction::IncreaseOctave, tr("Transpose up one octave") },
 		{ Configuration::ShortcutAction::PrevInstrument, tr("Previous instrument") },
 		{ Configuration::ShortcutAction::NextInstrument, tr("Next instrument") },
 		{ Configuration::ShortcutAction::MaskInstrument, tr("Mask instrument") },
@@ -223,7 +246,7 @@ ConfigurationDialog::ConfigurationDialog(std::weak_ptr<Configuration> config, st
 		QObject::connect(button, &QToolButton::clicked, seq, &QKeySequenceEdit::clear);
 		auto layout = widget->layout();
 		layout->setSpacing(0);
-		layout->setMargin(0);
+		layout->setContentsMargins(0, 0, 0, 0);
 		layout->addWidget(seq);
 		layout->addWidget(button);
 		ui->shortcutsTreeWidget->setItemWidget(item, 1, widget);
@@ -266,6 +289,7 @@ ConfigurationDialog::ConfigurationDialog(std::weak_ptr<Configuration> config, st
 	for (const auto& pair : configLocked->getCustomLayoutKeys()) {
 		customLayoutKeysMap_.at(pair.second)->setKeySequence(QKeySequence(gui_utils::utf8ToQString(pair.first)));
 	}
+	updateNoteNames();
 
 	// Emulation //
 	ui->emulatorComboBox->addItem("MAME YM2608", static_cast<int>(chip::OpnaEmulator::Mame));
@@ -288,14 +312,16 @@ ConfigurationDialog::ConfigurationDialog(std::weak_ptr<Configuration> config, st
 	}
 	on_audioApiComboBox_currentIndexChanged(ui->audioApiComboBox->currentText());
 
-	ui->realChipComboBox->addItem(tr("None"), static_cast<int>(RealChipInterface::NONE));
-	ui->realChipComboBox->addItem("SCCI", static_cast<int>(RealChipInterface::SCCI));
-	ui->realChipComboBox->addItem("C86CTL", static_cast<int>(RealChipInterface::C86CTL));
+	ui->realChipComboBox->addItem(tr("None"), static_cast<int>(RealChipInterfaceType::NONE));
+#ifdef USE_REAL_CHIP
+	ui->realChipComboBox->addItem("SCCI", static_cast<int>(RealChipInterfaceType::SCCI));
+	ui->realChipComboBox->addItem("C86CTL", static_cast<int>(RealChipInterfaceType::C86CTL));
+#endif
 	switch (configLocked->getRealChipInterface()) {
 	default:	// Fall through
-	case RealChipInterface::NONE:	ui->realChipComboBox->setCurrentIndex(0);	break;
-	case RealChipInterface::SCCI:	ui->realChipComboBox->setCurrentIndex(1);	break;
-	case RealChipInterface::C86CTL:	ui->realChipComboBox->setCurrentIndex(2);	break;
+	case RealChipInterfaceType::NONE:	ui->realChipComboBox->setCurrentIndex(0);	break;
+	case RealChipInterfaceType::SCCI:	ui->realChipComboBox->setCurrentIndex(1);	break;
+	case RealChipInterfaceType::C86CTL:	ui->realChipComboBox->setCurrentIndex(2);	break;
 	}
 
 	{
@@ -324,7 +350,7 @@ ConfigurationDialog::ConfigurationDialog(std::weak_ptr<Configuration> config, st
 	case 48000:	ui->sampleRateComboBox->setCurrentIndex(1);	break;
 	case 55466:	ui->sampleRateComboBox->setCurrentIndex(2);	break;
 	}
-	ui->bufferLengthHorizontalSlider->setStyle(new SliderStyle());
+	ui->bufferLengthHorizontalSlider->setStyle(SliderStyle::instance());
 	QObject::connect(ui->bufferLengthHorizontalSlider, &QSlider::valueChanged,
 					 this, [&](int value) {
 		ui->bufferLengthLabel->setText(QString::number(value) + "ms");
@@ -413,6 +439,9 @@ void ConfigurationDialog::on_ConfigurationDialog_accepted()
 	// Wave view
 	configLocked->setWaveViewFrameRate(ui->waveViewRateSpinBox->value());
 
+	// Note names
+	configLocked->setNotationSystem(NOTATION_SYSS[ui->noteNameComboBox->currentIndex()].ev);
+
 	// Keys
 	std::unordered_map<Configuration::ShortcutAction, std::string> shortcuts;
 	for (const auto& pair: shortcutsMap_) {
@@ -438,7 +467,7 @@ void ConfigurationDialog::on_ConfigurationDialog_accepted()
 	// Sound //
 	configLocked->setSoundDevice(ui->audioDeviceComboBox->currentText().toUtf8().toStdString());
 	configLocked->setSoundAPI(ui->audioApiComboBox->currentText().toUtf8().toStdString());
-	configLocked->setRealChipInterface(static_cast<RealChipInterface>(
+	configLocked->setRealChipInterface(static_cast<RealChipInterfaceType>(
 										   ui->realChipComboBox->currentData(Qt::UserRole).toInt()));
 	configLocked->setMidiEnabled(ui->midiInputGroupBox->isChecked());
 	configLocked->setMidiAPI(ui->midiApiComboBox->currentText().toUtf8().toStdString());
@@ -482,6 +511,22 @@ void ConfigurationDialog::on_generalSettingsListWidget_itemSelectionChanged()
 	ui->descPlainTextEdit->setPlainText(tr("Description: %1").arg(text));
 }
 
+void ConfigurationDialog::on_noteNameComboBox_currentIndexChanged(int index)
+{
+	// Change notation system temporary
+	NoteNameManager& man = NoteNameManager::getManager();
+	man.setNotationSystem(NOTATION_SYSS[index].ev);
+	updateNoteNames();
+
+	// Restore global notation system
+	for (const auto& attrib : NOTATION_SYSS) {
+		if (attrib.ev == config_.lock()->getNotationSystem()) {
+			man.setNotationSystem(attrib.ev);
+			return;
+		}
+	}
+}
+
 /***** Mixer *****/
 void ConfigurationDialog::on_mixerResetPushButton_clicked()
 {
@@ -577,10 +622,10 @@ void ConfigurationDialog::on_removeEnvelopeSetpushButton_clicked()
 void ConfigurationDialog::on_editEnvelopeSetPushButton_clicked()
 {
 	size_t row = static_cast<size_t>(ui->envelopeTypeListWidget->currentRow());
-	FMEnvelopeSetEditDialog diag(fmEnvelopeTexts_.at(row).texts);
-	diag.setWindowTitle(diag.windowTitle() + ": " + ui->envelopeSetNameLineEdit->text());
-	if (diag.exec() == QDialog::Accepted) {
-		fmEnvelopeTexts_.at(row).texts = diag.getSet();
+	FMEnvelopeSetEditDialog dialog(fmEnvelopeTexts_.at(row).texts, this);
+	dialog.setWindowTitle(dialog.windowTitle() + ": " + ui->envelopeSetNameLineEdit->text());
+	if (dialog.exec() == QDialog::Accepted) {
+		fmEnvelopeTexts_.at(row).texts = dialog.getSet();
 	}
 }
 
@@ -616,9 +661,8 @@ void ConfigurationDialog::updateEnvelopeSetUi()
 }
 
 /***** Keys *****/
-void ConfigurationDialog::on_keyboardTypeComboBox_currentIndexChanged(int index)
+void ConfigurationDialog::on_keyboardTypeComboBox_currentIndexChanged(int)
 {
-	Q_UNUSED(index)
 	bool enableCustomLayoutInterface = ui->keyboardTypeComboBox->currentIndex() == 0;
 	ui->lowHighKeysTabWidget->setEnabled(enableCustomLayoutInterface);
 	ui->customLayoutResetButton->setEnabled(enableCustomLayoutInterface);
@@ -650,26 +694,65 @@ std::string ConfigurationDialog::getShortcutString(int row) const
 				)->keySequence().toString().toStdString();
 }
 
+void ConfigurationDialog::updateNoteNames()
+{
+	const NoteNameManager& man = NoteNameManager::getManager();
+	ui->lowCLabel->setText(man.getNoteName(0));
+	ui->lowCSLabel->setText(man.getNoteName(1));
+	ui->lowDLabel->setText(man.getNoteName(2));
+	ui->lowDSLabel->setText(man.getNoteName(3));
+	ui->lowELabel->setText(man.getNoteName(4));
+	ui->lowFLabel->setText(man.getNoteName(5));
+	ui->lowFSLabel->setText(man.getNoteName(6));
+	ui->lowGLabel->setText(man.getNoteName(7));
+	ui->lowGSLabel->setText(man.getNoteName(8));
+	ui->lowALabel->setText(man.getNoteName(9));
+	ui->lowASLabel->setText(man.getNoteName(10));
+	ui->lowBLabel->setText(man.getNoteName(11));
+	ui->lowHighCLabel->setText(man.getNoteName(0));
+	ui->lowHighCSLabel->setText(man.getNoteName(1));
+	ui->lowHighDLabel->setText(man.getNoteName(2));
+	ui->highCLabel->setText(man.getNoteName(0));
+	ui->highCSLabel->setText(man.getNoteName(1));
+	ui->highDLabel->setText(man.getNoteName(2));
+	ui->highDSLabel->setText(man.getNoteName(3));
+	ui->highELabel->setText(man.getNoteName(4));
+	ui->highFLabel->setText(man.getNoteName(5));
+	ui->highFSLabel->setText(man.getNoteName(6));
+	ui->highGLabel->setText(man.getNoteName(7));
+	ui->highGSLabel->setText(man.getNoteName(8));
+	ui->highALabel->setText(man.getNoteName(9));
+	ui->highASLabel->setText(man.getNoteName(10));
+	ui->highBLabel->setText(man.getNoteName(11));
+	ui->highHighCLabel->setText(man.getNoteName(0));
+	ui->highHighCSLabel->setText(man.getNoteName(1));
+	ui->highHighDLabel->setText(man.getNoteName(2));
+}
+
 /***** Appearance *****/
 void ConfigurationDialog::on_colorEditPushButton_clicked()
 {
 	QTreeWidgetItem* item = ui->colorsTreeWidget->currentItem();
 	if (item == nullptr || item->parent() == nullptr) return;
-	QColorDialog diag(item->data(1, Qt::BackgroundRole).value<QColor>());
-	diag.setOption(QColorDialog::ShowAlphaChannel);
-	if (diag.exec() == QDialog::Accepted)
-		item->setData(1, Qt::BackgroundRole, diag.currentColor());
+	QColorDialog dialog(item->data(1, Qt::BackgroundRole).value<QColor>(), this);
+	dialog.setOption(QColorDialog::ShowAlphaChannel);
+	if (dialog.exec() == QDialog::Accepted)
+		item->setData(1, Qt::BackgroundRole, dialog.currentColor());
 }
 
 void ConfigurationDialog::on_colorLoadPushButton_clicked()
 {
 	QString file = QFileDialog::getOpenFileName(this, tr("Open color scheme"),
 												QApplication::applicationDirPath() + "/skins",
-												tr("ini file (*.ini)"));
+												tr("ini file (*.ini)") + ";;" + tr("All files (*)"), nullptr
+#if defined(Q_OS_LINUX) || (defined(Q_OS_BSD4) && !defined(Q_OS_DARWIN))
+												, QFileDialog::DontUseNativeDialog
+#endif
+												);
 	if (file.isNull()) return;
 
 	ColorPalette palette;
-	if (ColorPaletteHandler::loadPalette(file, &palette)) {
+	if (io::loadPalette(file, &palette)) {
 		updateColorTreeFrom(&palette);
 	}
 	else {
@@ -681,13 +764,17 @@ void ConfigurationDialog::on_colorSavePushButton_clicked()
 {
 	QString file = QFileDialog::getSaveFileName(this, tr("Save color scheme"),
 												QApplication::applicationDirPath() + "/skins",
-												tr("ini file (*.ini)"));
+												tr("ini file (*.ini)") + ";;" + tr("All files (*)"), nullptr
+#if defined(Q_OS_LINUX) || (defined(Q_OS_BSD4) && !defined(Q_OS_DARWIN))
+												, QFileDialog::DontUseNativeDialog
+#endif
+												);
 	if (file.isNull()) return;
 	if (!file.endsWith(".ini")) file += ".ini";	// For linux
 
 	ColorPalette palette;
 	setPaletteFromColorTree(&palette);
-	if (!ColorPaletteHandler::savePalette(file, &palette))
+	if (!io::savePalette(file, &palette))
 		QMessageBox::critical(this, tr("Error"), tr("Failed to save the color scheme."));
 }
 
