@@ -28,6 +28,7 @@
 #include <unordered_map>
 #include <QString>
 #include <QSettings>
+#include <QFile>
 #include "configuration.hpp"
 #include "jamming.hpp"
 #include "enum_hash.hpp"
@@ -39,8 +40,7 @@ namespace io
 namespace
 {
 // config path (*nix): ~/.config/<organization>/<application>.ini
-const QString ORG_NAME = "BambooTracker";
-const QString APP_NAME = "BambooTracker";
+const QString APPLICATION = "BambooTracker";
 
 const std::unordered_map<Configuration::ShortcutAction, QString> SHORTCUTS_NAME_MAP = {
 	{ Configuration::ShortcutAction::KeyOff, "keyOff" },
@@ -152,7 +152,7 @@ const std::unordered_map<JamKey, QString> JAM_KEY_NAME_MAP = {
 bool saveConfiguration(std::weak_ptr<Configuration> config)
 {
 	try {
-		QSettings settings(QSettings::IniFormat, QSettings::UserScope, ORG_NAME, APP_NAME);
+		QSettings settings(QSettings::IniFormat, QSettings::UserScope, io::ORGANIZATION_NAME, APPLICATION);
 		std::shared_ptr<Configuration> configLocked = config.lock();
 
 		// Internal //
@@ -171,6 +171,7 @@ bool saveConfiguration(std::weak_ptr<Configuration> config)
 		settings.setValue("instrumentADPCMWindowHeight", configLocked->getInstrumentADPCMWindowHeight());
 		settings.setValue("instrumentDrumkitWindowWidth",  configLocked->getInstrumentDrumkitWindowWidth());
 		settings.setValue("instrumentDrumkitWindowHeight", configLocked->getInstrumentDrumkitWindowHeight());
+		settings.setValue("instrumentDrumkitWindowHorizontalSplit", configLocked->getInstrumentDrumkitWindowHorizontalSplit());
 		settings.setValue("followMode",		configLocked->getFollowMode());
 		settings.setValue("workingDirectory",          QString::fromStdString(configLocked->getWorkingDirectory()));
 		settings.setValue("instrumentOpenFormat",		configLocked->getInstrumentOpenFormat());
@@ -231,6 +232,11 @@ bool saveConfiguration(std::weak_ptr<Configuration> config)
 		settings.setValue("frameRate", configLocked->getWaveViewFrameRate());
 		settings.endGroup();
 
+		// Note names
+		settings.beginGroup("NoteNames");
+		settings.setValue("notationSystem", static_cast<int>(configLocked->getNotationSystem()));
+		settings.endGroup();
+
 		// Keys
 		settings.beginGroup("Keys");
 		for (const auto& pair : configLocked->getShortcuts()) {
@@ -301,7 +307,9 @@ bool saveConfiguration(std::weak_ptr<Configuration> config)
 bool loadConfiguration(std::weak_ptr<Configuration> config)
 {
 	try {
-		QSettings settings(QSettings::IniFormat, QSettings::UserScope, ORG_NAME, APP_NAME);
+		QSettings settings(QSettings::IniFormat, QSettings::UserScope, io::ORGANIZATION_NAME, APPLICATION);
+		if (!QFile(settings.fileName()).exists()) return false;
+
 		std::shared_ptr<Configuration> configLocked = config.lock();
 
 		// Internal //
@@ -319,6 +327,7 @@ bool loadConfiguration(std::weak_ptr<Configuration> config)
 		configLocked->setInstrumentADPCMWindowWidth(settings.value("instrumentADPCMWindowWidth", configLocked->getInstrumentADPCMWindowWidth()).toInt());
 		configLocked->setInstrumentADPCMWindowHeight(settings.value("instrumentADPCMWindowHeight", configLocked->getInstrumentADPCMWindowHeight()).toInt());
 		configLocked->setInstrumentDrumkitWindowWidth(settings.value("instrumentDrumkitWindowWidth", configLocked->getInstrumentDrumkitWindowWidth()).toInt());
+		configLocked->setInstrumentDrumkitWindowHorizontalSplit(settings.value("instrumentDrumkitWindowHorizontalSplit", configLocked->getInstrumentDrumkitWindowHorizontalSplit()).toInt());
 		configLocked->setInstrumentDrumkitWindowHeight(settings.value("instrumentDrumkitWindowHeight", configLocked->getInstrumentDrumkitWindowHeight()).toInt());
 		configLocked->setFollowMode(settings.value("followMode", configLocked->getFollowMode()).toBool());
 		configLocked->setWorkingDirectory(settings.value("workingDirectory", QString::fromStdString(configLocked->getWorkingDirectory())).toString().toStdString());
@@ -392,6 +401,12 @@ bool loadConfiguration(std::weak_ptr<Configuration> config)
 		configLocked->setWaveViewFrameRate(settings.value("frameRate", configLocked->getWaveViewFrameRate()).toInt());
 		settings.endGroup();
 
+		// Note names
+		settings.beginGroup("NoteNames");
+		configLocked->setNotationSystem(
+					static_cast<NoteNotationSystem>(settings.value("notationSystem", static_cast<int>(configLocked->getNotationSystem())).toInt()));
+		settings.endGroup();
+
 		// Keys
 		settings.beginGroup("Keys");
 		std::unordered_map<Configuration::ShortcutAction, std::string> shortcuts;
@@ -421,7 +436,7 @@ bool loadConfiguration(std::weak_ptr<Configuration> config)
 		settings.beginGroup("Sound");
 		configLocked->setSoundAPI(settings.value("soundAPI", QString::fromStdString(configLocked->getSoundAPI())).toString().toUtf8().toStdString());
 		configLocked->setSoundDevice(settings.value("soundDevice", QString::fromStdString(configLocked->getSoundDevice())).toString().toUtf8().toStdString());
-		configLocked->setRealChipInterface(static_cast<RealChipInterface>(
+		configLocked->setRealChipInterface(static_cast<RealChipInterfaceType>(
 											   settings.value("realChipInterface", static_cast<int>(configLocked->getRealChipInterface())).toInt()));
 		configLocked->setEmulator(settings.value("emulator", configLocked->getEmulator()).toInt());
 		QVariant sampleRateWorkaround;
@@ -477,7 +492,8 @@ bool loadConfiguration(std::weak_ptr<Configuration> config)
 		settings.endGroup();
 
 		return true;
-	} catch (...) {
+	}
+	catch (...) {
 		return false;
 	}
 }
