@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2019-2021 Rerrah
+ * Copyright (C) 2019-2022 Rerrah
  *
  * Permission is hereby granted, free of charge, to any person
  * obtaining a copy of this software and associated documentation
@@ -34,7 +34,7 @@ EffectListDialog::EffectListDialog(QWidget *parent) :
 	ui(new Ui::EffectListDialog)
 {
 	ui->setupUi(this);
-	setWindowFlags(windowFlags() ^ Qt::WindowContextHelpButtonHint);
+	setWindowFlags(windowFlags() & ~Qt::WindowContextHelpButtonHint);
 
 	ui->tableWidget->setColumnWidth(0, 50);
 	ui->tableWidget->setColumnWidth(1, 100);
@@ -55,6 +55,7 @@ EffectListDialog::EffectListDialog(QWidget *parent) :
 	addRow(EffectType::AutoEnvelope, { SoundSource::SSG });
 	addRow(EffectType::HardEnvHighPeriod, { SoundSource::SSG });
 	addRow(EffectType::HardEnvLowPeriod, { SoundSource::SSG });
+	addRow(EffectType::Retrigger, { SoundSource::FM, SoundSource::SSG, SoundSource::RHYTHM, SoundSource::ADPCM });
 	addRow(EffectType::Groove, { SoundSource::FM, SoundSource::SSG, SoundSource::RHYTHM, SoundSource::ADPCM });
 	addRow(EffectType::Detune, { SoundSource::FM, SoundSource::SSG, SoundSource::ADPCM });
 	addRow(EffectType::NoteSlideUp, { SoundSource::FM, SoundSource::SSG, SoundSource::ADPCM });
@@ -70,6 +71,8 @@ EffectListDialog::EffectListDialog(QWidget *parent) :
 	addRow(EffectType::ARControl, { SoundSource::FM });
 	addRow(EffectType::Brightness, { SoundSource::FM });
 	addRow(EffectType::DRControl, { SoundSource::FM });
+	addRow(EffectType::XVolumeSlide, { SoundSource::FM, SoundSource::SSG, SoundSource::ADPCM });
+	addRow(EffectType::EnvelopeReset, { SoundSource::FM });
 	addRow(EffectType::FBControl, { SoundSource::FM });
 	addRow(EffectType::FineDetune, { SoundSource::FM, SoundSource::SSG, SoundSource::ADPCM });
 	addRow(EffectType::VolumeDelay, { SoundSource::FM, SoundSource::SSG, SoundSource::RHYTHM, SoundSource::ADPCM });
@@ -87,7 +90,7 @@ void EffectListDialog::addRow(EffectType effect, std::unordered_set<SoundSource>
 {
 	int row = ui->tableWidget->rowCount();
 	ui->tableWidget->insertRow(row);
-	ui->tableWidget->setItem(row, 0, new QTableWidgetItem(EffectDescription::getEffectFormat(effect)));
+	ui->tableWidget->setItem(row, 0, new QTableWidgetItem(effect_desc::getEffectFormat(effect)));
 	ui->tableWidget->setRowHeight(row, ui->tableWidget->horizontalHeader()->height());
 	QString typeStr("");
 	if (types.count(SoundSource::FM)) typeStr += "FM";
@@ -95,5 +98,5 @@ void EffectListDialog::addRow(EffectType effect, std::unordered_set<SoundSource>
 	if (types.count(SoundSource::RHYTHM)) typeStr = typeStr + (typeStr.isEmpty() ? "" : ", ") + tr("Rhythm");
 	if (types.count(SoundSource::ADPCM)) typeStr = typeStr + (typeStr.isEmpty() ? "" : ", ") + "ADPCM";
 	ui->tableWidget->setItem(row, 1, new QTableWidgetItem(typeStr));
-	ui->tableWidget->setItem(row, 2, new QTableWidgetItem(EffectDescription::getEffectDescription(effect)));
+	ui->tableWidget->setItem(row, 2, new QTableWidgetItem(effect_desc::getEffectDescription(effect)));
 }
