@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2018-2021 Rerrah
+ * Copyright (C) 2018-2022 Rerrah
  *
  * Permission is hereby granted, free of charge, to any person
  * obtaining a copy of this software and associated documentation
@@ -54,7 +54,7 @@
 #include "bamboo_tracker.hpp"
 #include "precise_timer.hpp"
 #include "audio/audio_stream.hpp"
-#include "gui/instrument_editor/instrument_form_manager.hpp"
+#include "gui/instrument_editor/instrument_editor_manager.hpp"
 #include "gui/color_palette.hpp"
 #include "gui/file_history.hpp"
 #include "gui/effect_list_dialog.hpp"
@@ -62,24 +62,34 @@
 #include "gui/bookmark_manager_form.hpp"
 #include "gui/instrument_selection_dialog.hpp"
 #include "gui/comment_edit_dialog.hpp"
+#include "gui/key_signature_manager_form.hpp"
 
 namespace Ui {
-	class MainWindow;
+class ModuleSaveCheckDialog;
+class MainWindow;
 }
 
+class ModuleSaveCheckDialog : public QMessageBox
+{
+	Q_OBJECT
+
+public:
+	explicit ModuleSaveCheckDialog(const std::string& name, QWidget* parent = nullptr);
+};
+
 class MainWindow : public QMainWindow
 {
 	Q_OBJECT
 
 public:
-	MainWindow(std::weak_ptr<Configuration> config, QString filePath, QWidget *parent = nullptr);
+	MainWindow(std::weak_ptr<Configuration> config, QString filePath, bool isFirstLaunch, QWidget *parent = nullptr);
 	~MainWindow() override;
 
 	void assignADPCMSamples();
 
 protected:
 	bool eventFilter(QObject* watched, QEvent* event) override;
-	void showEvent(QShowEvent* event) override;
+	void showEvent(QShowEvent*) override;
 	void keyPressEvent(QKeyEvent* event) override;
 	void keyReleaseEvent(QKeyEvent* event) override;
 	void dragEnterEvent(QDragEnterEvent* event) override;
@@ -106,10 +116,10 @@ private:
 	std::shared_ptr<QUndoStack> comStack_;
 	std::shared_ptr<FileHistory> fileHistory_;
 
-	std::unique_ptr<QLibrary> scciDll_, c86ctlDll_;
+	std::shared_ptr<QLibrary> scciDll_, c86ctlDll_;
 
 	// Instrument list
-	std::shared_ptr<InstrumentFormManager> instForms_;
+	std::shared_ptr<InstrumentEditorManager> instDialogMan_;
 	QListWidgetItem* renamingInstItem_;
 	QLineEdit* renamingInstEdit_;
 	void addInstrument();
@@ -136,7 +146,7 @@ private:
 
 	// Load data
 	void loadModule();
-	void openModule(QString file);
+	void openModule(const QString &file);
 	void loadSong();
 
 	// Play song
@@ -155,7 +165,7 @@ private:
 
 	// Configuration change
 	void changeConfiguration();
-	void setRealChipInterface(RealChipInterface intf);
+	void setRealChipInterface(RealChipInterfaceType intf);
 	void setMidiConfiguration();
 	void updateFonts();
 
@@ -169,7 +179,6 @@ private:
 	void setModifiedTrue();
 	void setInitialSelectedInstrument();
 	QString getModuleFileBaseName() const;
-	int getSelectedFileFilter(QString& file, QStringList& filters) const;
 
 	bool isEditedPattern_, isEditedOrder_, isEditedInstList_;
 	bool isSelectedPattern_, isSelectedOrder_;
@@ -187,7 +196,7 @@ private:
 	QSpinBox *octave_, *highlight1_, *highlight2_, *volume_;
 
 	// Status bars
-	QLabel *statusDetail_, *statusStyle_, *statusInst_, *statusOctave_;
+	QLabel *statusStyle_, *statusInst_, *statusOctave_;
 	QLabel *statusIntr_, *statusMixer_, *statusBpm_, *statusPlayPos_;
 
 	// Shortcuts
@@ -200,17 +209,16 @@ private:
 	void setShortcuts();
 
 	// Dialogs
-	std::unique_ptr<EffectListDialog> effListDiag_;
-	std::unique_ptr<KeyboardShortcutListDialog> shortcutsDiag_;
+	std::unique_ptr<EffectListDialog> effListDialog_;
+	std::unique_ptr<KeyboardShortcutListDialog> shortcutsDialog_;
 	std::unique_ptr<BookmarkManagerForm> bmManForm_;
-	std::unique_ptr<CommentEditDialog> commentDiag_;
+	std::unique_ptr<CommentEditDialog> commentDialog_;
+	std::unique_ptr<KeySignatureManagerForm> ksManForm_;
+	std::unique_ptr<QMessageBox> welcomeDialog_;
 
 	// Bank import
 	std::atomic_bool bankJamMidiCtrl_;
-	std::unique_ptr<InstrumentSelectionDialog> importBankDiag_;
-
-	// Track visibility
-	void setTrackVisibility(const std::vector<int>& visTracks);
+	std::unique_ptr<InstrumentSelectionDialog> importBankDialog_;
 
 	// Meta methods
 	int tickEventMethod_;
@@ -256,7 +264,7 @@ private:
 
 private slots:
 	void on_instrumentList_customContextMenuRequested(const QPoint& pos);
-	void on_instrumentList_itemDoubleClicked(QListWidgetItem* item);
+	void on_instrumentList_itemDoubleClicked(QListWidgetItem*);
 	void on_instrumentList_itemSelectionChanged();
 	void on_grooveCheckBox_stateChanged(int arg1);
 	void on_actionExit_triggered();
@@ -361,6 +369,8 @@ private slots:
 	void on_action_Insert_triggered();
 	void on_action_Hide_Tracks_triggered();
 	void on_action_Estimate_Song_Length_triggered();
+	void on_action_Key_Signature_Manager_triggered();
+	void on_action_Welcome_triggered();
 };
 
 #endif // MAINWINDOW_HPP
