@@ -37,16 +37,24 @@
 #include <QMenu>
 #include <QAction>
 #include <QPoint>
+#if QT_VERSION >= QT_VERSION_CHECK(6, 0, 0)
+#include <QPointF>
+#endif
 #include <QString>
 #include <QIcon>
 #include "playback.hpp"
 #include "track.hpp"
 #include "bamboo_tracker_defs.hpp"
+#include "gui/dpi.hpp"
 #include "gui/event_guard.hpp"
 #include "gui/command/order/order_commands_qt.hpp"
 #include "gui/gui_utils.hpp"
 #include "utils.hpp"
 
+using Dpi::scaledQPixmap;
+using Dpi::iRatio;
+using Dpi::scaleRect;
+
 OrderListPanel::OrderListPanel(QWidget *parent)
 	: QWidget(parent),
 	  config_(std::make_shared<Configuration>()),	// Dummy
@@ -93,7 +101,7 @@ OrderListPanel::OrderListPanel(QWidget *parent)
 	  repaintingCnt_(0),
 	  playingRow_(-1),
 	  insSc1_(Qt::Key_Insert, this, nullptr, nullptr, Qt::WidgetShortcut),
-	  insSc2_(Qt::ALT + Qt::Key_B, this, nullptr, nullptr, Qt::WidgetShortcut),
+	  insSc2_(Qt::ALT | Qt::Key_B, this, nullptr, nullptr, Qt::WidgetShortcut),
 	  menuSc_(Qt::Key_Menu, this, nullptr, nullptr, Qt::WidgetShortcut)
 {
 	setAttribute(Qt::WA_Hover);
@@ -105,7 +113,7 @@ OrderListPanel::OrderListPanel(QWidget *parent)
 	headerFont_.setPointSize(10);
 	rowFont_ = QFont("Monospace", 10);
 	rowFont_.setStyleHint(QFont::TypeWriter);
-	rowFont_.setStyleStrategy(QFont::ForceIntegerMetrics);
+	rowFont_.setStyleStrategy(QFont::PreferMatch);
 
 	updateSizes();
 
@@ -255,6 +263,7 @@ void OrderListPanel::updateSizes()
 void OrderListPanel::initDisplay()
 {
 	int width = geometry().width();
+	int ratio = iRatio(*this);
 
 	// Recalculate pixmap sizes
 	viewedRegionHeight_ = std::max((geometry().height() - headerHeight_), rowFontHeight_);
@@ -266,10 +275,10 @@ void OrderListPanel::initDisplay()
 	viewedCenterY_ = (viewedRowsHeight_ - rowFontHeight_) >> 1;
 	viewedCenterBaseY_ = viewedCenterY_ + rowFontAscent_ + (rowFontLeading_ >> 1);
 
-	completePixmap_ = QPixmap(geometry().size());
-	backPixmap_ = QPixmap(width, viewedRowsHeight_);
-	textPixmap_ = QPixmap(width, viewedRowsHeight_);
-	headerPixmap_ = QPixmap(width, headerHeight_);
+	completePixmap_ = scaledQPixmap(geometry().size(), ratio);
+	backPixmap_ = scaledQPixmap(width, viewedRowsHeight_, ratio);
+	textPixmap_ = scaledQPixmap(width, viewedRowsHeight_, ratio);
+	headerPixmap_ = scaledQPixmap(width, headerHeight_, ratio);
 }
 
 void OrderListPanel::drawList(const QRect &rect)
@@ -279,6 +288,7 @@ void OrderListPanel::drawList(const QRect &rect)
 		++repaintingCnt_;	// Use module data after this line
 
 		if (backChanged_ || textChanged_ || headerChanged_ || orderDownCount_ || followModeChanged_) {
+			int ratio = iRatio(*this);
 
 			int maxWidth = std::min(geometry().width(), columnsWidthFromLeftToEnd_);
 			completePixmap_.fill(palette_->odrBackColor);
@@ -301,10 +311,11 @@ void OrderListPanel::drawList(const QRect &rect)
 			{
 				QPainter mergePainter(&completePixmap_);
 				QRect rowsRect(0, viewedRowOffset_, maxWidth, viewedRegionHeight_);
+				rowsRect = scaleRect(rowsRect, ratio);
 				QRect inViewRect(0, headerHeight_, maxWidth, viewedRegionHeight_);
 				mergePainter.drawPixmap(inViewRect, backPixmap_, rowsRect);
 				mergePainter.drawPixmap(inViewRect, textPixmap_, rowsRect);
-				mergePainter.drawPixmap(headerPixmap_.rect(), headerPixmap_);
+				mergePainter.drawPixmap(QPoint(0, 0), headerPixmap_);
 			}
 
 			if (!hasFocus()) drawShadow();
@@ -470,13 +481,22 @@ void OrderListPanel::drawRows(int maxWidth)
 
 void OrderListPanel::quickDrawRows(int maxWidth)
 {
+	int ratio = iRatio(*this);
+
 	int halfRowsCnt = viewedRowCnt_ >> 1;
 	int shift = rowFontHeight_ * orderDownCount_;
 
 	/* Move up by */
-	QRect srcRect(0, 0, maxWidth, viewedRowsHeight_);
-	textPixmap_.scroll(0, -shift, srcRect);
-	backPixmap_.scroll(0, -shift, srcRect);
+	{
+		// QPixmap::scroll() takes physical pixels, not virtual.
+		int phShift = shift * ratio;
+		QRect srcRect(0, 0, maxWidth, viewedRowsHeight_);
+		srcRect = scaleRect(srcRect, ratio);
+
+		textPixmap_.scroll(0, -phShift, srcRect);
+		backPixmap_.scroll(0, -phShift, srcRect);
+	}
+
 	{
 		int fpos = viewedCenterPos_.row + orderDownCount_ - halfRowsCnt;
 		if (fpos >= 0) viewedFirstPos_.row = fpos;
@@ -621,13 +641,16 @@ void OrderListPanel::quickDrawRows(int maxWidth)
 
 void OrderListPanel::drawHeaders(int maxWidth)
 {
+	static const QString RHYTM_NAMES[6] = {
+		"BD", "SD", "TOP", "HH", "TOM", "RIM"
+	};
 	QPainter painter(&headerPixmap_);
 	painter.setFont(headerFont_);
 
 	painter.fillRect(0, 0, geometry().width(), headerHeight_, palette_->odrHeaderRowColor);
 	painter.setPen(palette_->odrHeaderBorderColor);
-	int bottomLineY = headerHeight_ - 1;
-	painter.drawLine(0, bottomLineY, geometry().width(), bottomLineY);
+	qreal bottomLineY = headerHeight_ - 0.5;
+	painter.drawLine(QPointF(0., bottomLineY), QPointF(geometry().width(), bottomLineY));
 
 	for (int x = rowNumWidth_, trackVisIdx = leftTrackVisIdx_; x < maxWidth; ++trackVisIdx) {
 		painter.setPen(palette_->odrHeaderBorderColor);
@@ -655,9 +678,6 @@ void OrderListPanel::drawHeaders(int maxWidth)
 			str = "SG" + QString::number(attrib.channelInSource + 1);
 			break;
 		case SoundSource::RHYTHM:
-			static const QString RHYTM_NAMES[6] = {
-				"BD", "SD", "TOP", "HH", "TOM", "RIM"
-			};
 			str = RHYTM_NAMES[attrib.channelInSource];
 			break;
 		case SoundSource::ADPCM:
@@ -1012,6 +1032,25 @@ void OrderListPanel::pasteCopiedCells(const OrderPosition& startPos)
 	comStack_.lock()->push(new PasteCopiedDataToOrderQtCommand(this));
 }
 
+void OrderListPanel::clonePatterns(const OrderPosition& singlePos)
+{
+	int bo, bt, eo, et;
+	if (selLeftAbovePos_.row != -1) {
+		bo = selLeftAbovePos_.row;
+		bt = visTracks_.at(selLeftAbovePos_.trackVisIdx);
+		eo = selRightBelowPos_.row;
+		et = visTracks_.at(selRightBelowPos_.trackVisIdx);
+	}
+	else if (singlePos.row >= 0 && singlePos.trackVisIdx >= 0) {
+		bo = eo = singlePos.row;
+		bt = et = visTracks_.at(singlePos.trackVisIdx);
+	}
+	else return;
+
+	bt_->clonePatterns(curSongNum_, bo, bt, eo, et);
+	comStack_.lock()->push(new ClonePatternsQtCommand(this));
+}
+
 void OrderListPanel::setSelectedRectangle(const OrderPosition& start, const OrderPosition& end)
 {
 	if (start.trackVisIdx > end.trackVisIdx) {
@@ -1053,31 +1092,31 @@ void OrderListPanel::showContextMenu(const OrderPosition& pos, const QPoint& poi
 	// Leave Before Qt5.7.0 style due to windows xp
 	QAction* insert = menu.addAction(tr("&Insert Order"));
 	insert->setIcon(QIcon(":/icon/insert_order"));
-	QObject::connect(insert, &QAction::triggered, this, [&]() { insertOrderBelow(); });
+	QObject::connect(insert, &QAction::triggered, this, [&] { insertOrderBelow(); });
 	QAction* remove = menu.addAction(tr("&Remove Order"));
 	remove->setIcon(QIcon(":/icon/remove_order"));
-	QObject::connect(remove, &QAction::triggered, this, [&]() { deleteOrder(); });
+	QObject::connect(remove, &QAction::triggered, this, [&] { deleteOrder(); });
 	QAction* duplicate = menu.addAction(tr("&Duplicate Order"));
 	duplicate->setIcon(QIcon(":/icon/duplicate_order"));
 	QObject::connect(duplicate, &QAction::triggered, this, &OrderListPanel::onDuplicatePressed);
 	QAction* clonep = menu.addAction(tr("&Clone Patterns"));
-	QAction::connect(clonep, &QAction::triggered, this, &OrderListPanel::onClonePatternsPressed);
+	QAction::connect(clonep, &QAction::triggered, this, [&, pos] { clonePatterns(pos); });
 	QAction* cloneo = menu.addAction(tr("Clone &Order"));
 	QObject::connect(cloneo, &QAction::triggered, this, &OrderListPanel::onCloneOrderPressed);
 	menu.addSeparator();
 	QAction* moveUp = menu.addAction(tr("Move Order &Up"));
 	moveUp->setIcon(QIcon(":/icon/order_up"));
-	QObject::connect(moveUp, &QAction::triggered, this, [&]() { onMoveOrderPressed(true); });
+	QObject::connect(moveUp, &QAction::triggered, this, [&] { onMoveOrderPressed(true); });
 	QAction* moveDown = menu.addAction(tr("Move Order Do&wn"));
 	moveDown->setIcon(QIcon(":/icon/order_down"));
-	QObject::connect(moveDown, &QAction::triggered, this, [&]() { onMoveOrderPressed(false); });
+	QObject::connect(moveDown, &QAction::triggered, this, [&] { onMoveOrderPressed(false); });
 	menu.addSeparator();
 	QAction* copy = menu.addAction(tr("Cop&y"));
 	copy->setIcon(QIcon(":/icon/copy"));
 	QObject::connect(copy, &QAction::triggered, this, &OrderListPanel::copySelectedCells);
 	QAction* paste = menu.addAction(tr("&Paste"));
 	paste->setIcon(QIcon(":/icon/paste"));
-	QObject::connect(paste, &QAction::triggered, this, [&]() { pasteCopiedCells(pos); });
+	QObject::connect(paste, &QAction::triggered, this, [&] { pasteCopiedCells(pos); });
 #if QT_VERSION >= QT_VERSION_CHECK(5, 10, 0)
 	duplicate->setShortcutVisibleInContextMenu(true);
 	clonep->setShortcutVisibleInContextMenu(true);
@@ -1088,10 +1127,11 @@ void OrderListPanel::showContextMenu(const OrderPosition& pos, const QPoint& poi
 	duplicate->setShortcut(gui_utils::strToKeySeq(shortcuts.at(Configuration::ShortcutAction::DuplicateOrder)));
 	clonep->setShortcut(gui_utils::strToKeySeq(shortcuts.at(Configuration::ShortcutAction::ClonePatterns)));
 	cloneo->setShortcut(gui_utils::strToKeySeq(shortcuts.at(Configuration::ShortcutAction::CloneOrder)));
-	copy->setShortcut(QKeySequence(Qt::CTRL + Qt::Key_C));
-	paste->setShortcut(QKeySequence(Qt::CTRL + Qt::Key_V));
+	copy->setShortcut(QKeySequence(Qt::CTRL | Qt::Key_C));
+	paste->setShortcut(QKeySequence(Qt::CTRL | Qt::Key_V));
 
-	if (pos.row < 0 || pos.trackVisIdx < 0) {
+	bool notCurHov = (pos.row < 0 || pos.trackVisIdx < 0);
+	if (notCurHov) {
 		remove->setEnabled(false);
 		moveUp->setEnabled(false);
 		moveDown->setEnabled(false);
@@ -1115,8 +1155,9 @@ void OrderListPanel::showContextMenu(const OrderPosition& pos, const QPoint& poi
 	}
 	if (selRightBelowPos_.row < 0
 			|| !isSelectedCell(pos.trackVisIdx, pos.row)) {
-		clonep->setEnabled(false);
 		copy->setEnabled(false);
+		// Turn off when no pattern is hilighted
+		if (notCurHov) clonep->setEnabled(false);
 	}
 	if (pos.row == 0) {
 		moveUp->setEnabled(false);
@@ -1291,11 +1332,7 @@ void OrderListPanel::onMoveOrderPressed(bool isUp)
 
 void OrderListPanel::onClonePatternsPressed()
 {
-	if (selLeftAbovePos_.row == -1) return;
-
-	bt_->clonePatterns(curSongNum_, selLeftAbovePos_.row, visTracks_.at(selLeftAbovePos_.trackVisIdx),
-					   selRightBelowPos_.row, visTracks_.at(selRightBelowPos_.trackVisIdx));
-	comStack_.lock()->push(new ClonePatternsQtCommand(this));
+	clonePatterns(curPos_);
 }
 
 void OrderListPanel::onCloneOrderPressed()
@@ -1472,8 +1509,6 @@ void OrderListPanel::resizeEvent(QResizeEvent* event)
 
 void OrderListPanel::mousePressEvent(QMouseEvent* event)
 {
-	Q_UNUSED(event)
-
 	mousePressPos_ = hovPos_;
 	mouseReleasePos_ = { -1, -1 };
 
@@ -1494,22 +1529,27 @@ void OrderListPanel::mouseMoveEvent(QMouseEvent* event)
 			setSelectedRectangle(mousePressPos_, hovPos_);
 		}
 
-		if (event->x() < rowNumWidth_ && leftTrackVisIdx_ > 0) {
+#if QT_VERSION >= QT_VERSION_CHECK(6, 0, 0)
+		QPointF pos = event->position();
+#else
+		QPoint pos = event->pos();
+#endif
+		if (pos.x() < rowNumWidth_ && leftTrackVisIdx_ > 0) {
 			if (config_->getMoveCursorByHorizontalScroll())
 				moveCursorToRight(-1);
 			else
 				moveViewToRight(-1);
 		}
-		else if (event->x() > geometry().width() - rowNumWidth_ && hovPos_.trackVisIdx != -1) {
+		else if (pos.x() > geometry().width() - rowNumWidth_ && hovPos_.trackVisIdx != -1) {
 			if (config_->getMoveCursorByHorizontalScroll())
 				moveCursorToRight(1);
 			else
 				moveViewToRight(1);
 		}
-		if (event->pos().y() < headerHeight_ + rowFontHeight_) {
+		if (pos.y() < headerHeight_ + rowFontHeight_) {
 			if (!bt_->isPlaySong() || !bt_->isFollowPlay()) moveCursorToDown(-1);
 		}
-		else if (event->pos().y() > geometry().height() - rowFontHeight_) {
+		else if (pos.y() > geometry().height() - rowFontHeight_) {
 			if (!bt_->isPlaySong() || !bt_->isFollowPlay()) moveCursorToDown(1);
 		}
 	}
@@ -1567,7 +1607,11 @@ void OrderListPanel::mouseReleaseEvent(QMouseEvent* event)
 
 bool OrderListPanel::mouseHoverd(QHoverEvent *event)
 {
+#if QT_VERSION >= QT_VERSION_CHECK(6, 0, 0)
+	QPointF pos = event->position();
+#else
 	QPoint pos = event->pos();
+#endif
 
 	OrderPosition oldPos = hovPos_;
 
@@ -1577,11 +1621,11 @@ bool OrderListPanel::mouseHoverd(QHoverEvent *event)
 	}
 	else {
 		if (pos.y() < curRowY_) {
-			int tmp = curPos_.row + (pos.y() - curRowY_) / rowFontHeight_ - 1;
+			int tmp = curPos_.row + static_cast<int>(std::ceil((pos.y() - curRowY_) / rowFontHeight_)) - 1;
 			hovPos_.row = (tmp < 0) ? -1 : tmp;
 		}
 		else {
-			hovPos_.row = curPos_.row + (pos.y() - curRowY_) / rowFontHeight_;
+			hovPos_.row = curPos_.row + static_cast<int>(std::floor((pos.y() - curRowY_) / rowFontHeight_));
 			if (hovPos_.row >= static_cast<int>(bt_->getOrderSize(curSongNum_))) hovPos_.row = -1;
 		}
 	}
@@ -1619,9 +1663,8 @@ void OrderListPanel::wheelEvent(QWheelEvent *event)
 	moveCursorToDown(-degree / 15);
 }
 
-void OrderListPanel::leaveEvent(QEvent* event)
+void OrderListPanel::leaveEvent(QEvent*)
 {
-	Q_UNUSED(event)
 	// Clear mouse hover selection
 	hovPos_ = { -1, -1 };
 }
