@@ -81,7 +81,7 @@ size_t loadInstrumentPropertyOperatorSequenceForInstrument(
 		uint16_t pos = ctr.readUint16(csr);
 		csr += 2;
 		// Release point check (prevents a bug)
-		// https://github.com/rerrahkr/BambooTracker/issues/11
+		// https://github.com/BambooTracker/BambooTracker/issues/11
 		if (pos < seqLen) instManLocked->setOperatorSequenceFMRelease(param, idx, InstrumentSequenceRelease(InstrumentSequenceRelease::FixedRelease, pos));
 		else instManLocked->setOperatorSequenceFMRelease(param, idx, InstrumentSequenceRelease(InstrumentSequenceRelease::NoRelease));
 		break;
@@ -182,6 +182,9 @@ AbstractInstrument* BtiIO::load(const BinaryContainer& ctr, const std::string& f
 				else kitSampFileMap[samp] = { key };
 				kit->setSampleEnabled(key, true);
 				kit->setPitch(key, ctr.readInt8(instCsr++));
+				if (fileVersion >= Version::toBCD(1, 5, 0)) {
+					kit->setPan(key, ctr.readUint8(instCsr++));
+				}
 			}
 			break;
 		}
@@ -500,6 +503,13 @@ AbstractInstrument* BtiIO::load(const BinaryContainer& ctr, const std::string& f
 					instPropCsr += ctr.readUint16(instPropCsr);
 					break;
 				}
+				case 0x2a:	// FM pan
+				{
+					nums.push_back(instManLocked->findFirstAssignablePanFM());
+					if (nums.back() == -1) throw FileCorruptionError(FileType::Inst, instPropCsr);
+					instPropCsr += ctr.readUint16(instPropCsr);
+					break;
+				}
 				case 0x30:	// SSG waveform
 				{
 					nums.push_back(instManLocked->findFirstAssignableWaveformSSG());
@@ -543,7 +553,7 @@ AbstractInstrument* BtiIO::load(const BinaryContainer& ctr, const std::string& f
 					if (inst->getType() == InstrumentType::Drumkit) {
 						kitSampMap[adpcmSampIdx] = kitSampFileMap.at(kitSampCnt++);
 					}
-					instPropCsr += ctr.readUint16(instPropCsr);
+					instPropCsr += ctr.readUint32(instPropCsr);
 					++adpcmSampIdx; // Increment for search appropriate kit sample
 					break;
 				}
@@ -568,6 +578,13 @@ AbstractInstrument* BtiIO::load(const BinaryContainer& ctr, const std::string& f
 					instPropCsr += ctr.readUint16(instPropCsr);
 					break;
 				}
+				case 0x44:	// ADPCM pan
+				{
+					nums.push_back(instManLocked->findFirstAssignablePanADPCM());
+					if (nums.back() == -1) throw FileCorruptionError(FileType::Inst, instPropCsr);
+					instPropCsr += ctr.readUint16(instPropCsr);
+					break;
+				}
 				default:
 					throw FileCorruptionError(FileType::Inst, instPropCsr);
 				}
@@ -952,7 +969,7 @@ AbstractInstrument* BtiIO::load(const BinaryContainer& ctr, const std::string& f
 						uint16_t pos = ctr.readUint16(csr);
 						csr += 2;
 						// Release point check (prevents a bug)
-						// https://github.com/rerrahkr/BambooTracker/issues/11
+						// https://github.com/BambooTracker/BambooTracker/issues/11
 						if (pos < seqLen) instManLocked->setArpeggioFMRelease(idx, InstrumentSequenceRelease(InstrumentSequenceRelease::FixedRelease, pos));
 						else instManLocked->setArpeggioFMRelease(idx, InstrumentSequenceRelease(InstrumentSequenceRelease::NoRelease));
 						break;
@@ -975,7 +992,7 @@ AbstractInstrument* BtiIO::load(const BinaryContainer& ctr, const std::string& f
 						default:
 							if (fileVersion < Version::toBCD(1, 2, 3)) {
 								// Recover deep clone bug
-								// https://github.com/rerrahkr/BambooTracker/issues/170
+								// https://github.com/BambooTracker/BambooTracker/issues/170
 								instManLocked->setArpeggioFMType(idx, SequenceType::AbsoluteSequence);
 								break;
 							}
@@ -1042,7 +1059,7 @@ AbstractInstrument* BtiIO::load(const BinaryContainer& ctr, const std::string& f
 						uint16_t pos = ctr.readUint16(csr);
 						csr += 2;
 						// Release point check (prevents a bug)
-						// https://github.com/rerrahkr/BambooTracker/issues/11
+						// https://github.com/BambooTracker/BambooTracker/issues/11
 						if (pos < seqLen) instManLocked->setPitchFMRelease(idx, InstrumentSequenceRelease(InstrumentSequenceRelease::FixedRelease, pos));
 						else instManLocked->setPitchFMRelease(idx, InstrumentSequenceRelease(InstrumentSequenceRelease::NoRelease));
 						break;
@@ -1062,7 +1079,7 @@ AbstractInstrument* BtiIO::load(const BinaryContainer& ctr, const std::string& f
 						default:
 							if (fileVersion < Version::toBCD(1, 2, 3)) {
 								// Recover deep clone bug
-								// https://github.com/rerrahkr/BambooTracker/issues/170
+								// https://github.com/BambooTracker/BambooTracker/issues/170
 								instManLocked->setPitchFMType(idx, SequenceType::AbsoluteSequence);
 								break;
 							}
@@ -1075,6 +1092,58 @@ AbstractInstrument* BtiIO::load(const BinaryContainer& ctr, const std::string& f
 					instPropCsr += ofs;
 					break;
 				}
+				case 0x2a:	// FM pan
+				{
+					int idx = *numIt++;
+					auto fm = dynamic_cast<InstrumentFM*>(inst);
+					fm->setPanEnabled(true);
+					fm->setPanNumber(idx);
+					uint16_t ofs = ctr.readUint16(instPropCsr);
+					size_t csr = instPropCsr + 2;
+
+					uint16_t seqLen = ctr.readUint16(csr);
+					csr += 2;
+					for (uint16_t l = 0; l < seqLen; ++l) {
+						uint16_t data = ctr.readUint16(csr);
+						csr += 2;
+						if (l == 0)
+							instManLocked->setPanFMSequenceData(idx, 0, data);
+						else
+							instManLocked->addPanFMSequenceData(idx, data);
+					}
+
+					uint16_t loopCnt = ctr.readUint16(csr);
+					csr += 2;
+					for (uint16_t l = 0; l < loopCnt; ++l) {
+						int begin = ctr.readUint16(csr);
+						csr += 2;
+						int end = ctr.readUint16(csr);
+						csr += 2;
+						int times = ctr.readUint8(csr++);
+						instManLocked->addPanFMLoop(idx, InstrumentSequenceLoop(begin, end, times));
+					}
+
+					switch (ctr.readUint8(csr++)) {
+					case 0x00:	// No release
+						instManLocked->setPanFMRelease(idx, InstrumentSequenceRelease(InstrumentSequenceRelease::NoRelease));
+						break;
+					case 0x01:	// Fixed
+					{
+						uint16_t pos = ctr.readUint16(csr);
+						csr += 2;
+						// Release point check (prevents a bug)
+						// https://github.com/BambooTracker/BambooTracker/issues/11
+						if (pos < seqLen) instManLocked->setPanFMRelease(idx, InstrumentSequenceRelease(InstrumentSequenceRelease::FixedRelease, pos));
+						else instManLocked->setPanFMRelease(idx, InstrumentSequenceRelease(InstrumentSequenceRelease::NoRelease));
+						break;
+					}
+					default:
+						throw FileCorruptionError(FileType::Inst, csr);
+					}
+
+					instPropCsr += ofs;
+					break;
+				}
 				case 0x30:	// SSG waveform
 				{
 					int idx = *numIt++;
@@ -1141,7 +1210,7 @@ AbstractInstrument* BtiIO::load(const BinaryContainer& ctr, const std::string& f
 						uint16_t pos = ctr.readUint16(csr);
 						csr += 2;
 						// Release point check (prevents a bug)
-						// https://github.com/rerrahkr/BambooTracker/issues/11
+						// https://github.com/BambooTracker/BambooTracker/issues/11
 						if (pos < seqLen) instManLocked->setWaveformSSGRelease(idx, InstrumentSequenceRelease(InstrumentSequenceRelease::FixedRelease, pos));
 						else instManLocked->setWaveformSSGRelease(idx, InstrumentSequenceRelease(InstrumentSequenceRelease::NoRelease));
 						break;
@@ -1204,7 +1273,7 @@ AbstractInstrument* BtiIO::load(const BinaryContainer& ctr, const std::string& f
 						uint16_t pos = ctr.readUint16(csr);
 						csr += 2;
 						// Release point check (prevents a bug)
-						// https://github.com/rerrahkr/BambooTracker/issues/11
+						// https://github.com/BambooTracker/BambooTracker/issues/11
 						if (pos < seqLen) instManLocked->setToneNoiseSSGRelease(idx, InstrumentSequenceRelease(InstrumentSequenceRelease::FixedRelease, pos));
 						else instManLocked->setToneNoiseSSGRelease(idx,  InstrumentSequenceRelease(InstrumentSequenceRelease::NoRelease));
 						break;
@@ -1267,7 +1336,7 @@ AbstractInstrument* BtiIO::load(const BinaryContainer& ctr, const std::string& f
 						instManLocked->setEnvelopeSSGRelease(idx, InstrumentSequenceRelease(InstrumentSequenceRelease::NoRelease));
 						break;
 						// Release point check (prevents a bug)
-						// https://github.com/rerrahkr/BambooTracker/issues/11
+						// https://github.com/BambooTracker/BambooTracker/issues/11
 					case 0x01:	// Fixed
 					{
 						uint16_t pos = ctr.readUint16(csr);
@@ -1344,7 +1413,7 @@ AbstractInstrument* BtiIO::load(const BinaryContainer& ctr, const std::string& f
 						uint16_t pos = ctr.readUint16(csr);
 						csr += 2;
 						// Release point check (prevents a bug)
-						// https://github.com/rerrahkr/BambooTracker/issues/11
+						// https://github.com/BambooTracker/BambooTracker/issues/11
 						if (pos < seqLen) instManLocked->setArpeggioSSGRelease(idx, InstrumentSequenceRelease(InstrumentSequenceRelease::FixedRelease, pos));
 						else instManLocked->setArpeggioSSGRelease(idx, InstrumentSequenceRelease(InstrumentSequenceRelease::NoRelease));
 						break;
@@ -1367,7 +1436,7 @@ AbstractInstrument* BtiIO::load(const BinaryContainer& ctr, const std::string& f
 						default:
 							if (fileVersion < Version::toBCD(1, 2, 3)) {
 								// Recover deep clone bug
-								// https://github.com/rerrahkr/BambooTracker/issues/170
+								// https://github.com/BambooTracker/BambooTracker/issues/170
 								instManLocked->setArpeggioSSGType(idx, SequenceType::AbsoluteSequence);
 								break;
 							}
@@ -1421,7 +1490,7 @@ AbstractInstrument* BtiIO::load(const BinaryContainer& ctr, const std::string& f
 						uint16_t pos = ctr.readUint16(csr);
 						csr += 2;
 						// Release point check (prevents a bug)
-						// https://github.com/rerrahkr/BambooTracker/issues/11
+						// https://github.com/BambooTracker/BambooTracker/issues/11
 						if (pos < seqLen) instManLocked->setPitchSSGRelease(idx, InstrumentSequenceRelease(InstrumentSequenceRelease::FixedRelease, pos));
 						else instManLocked->setPitchSSGRelease(idx, InstrumentSequenceRelease(InstrumentSequenceRelease::NoRelease));
 						break;
@@ -1441,7 +1510,7 @@ AbstractInstrument* BtiIO::load(const BinaryContainer& ctr, const std::string& f
 						default:
 							if (fileVersion < Version::toBCD(1, 2, 3)) {
 								// Recover deep clone bug
-								// https://github.com/rerrahkr/BambooTracker/issues/170
+								// https://github.com/BambooTracker/BambooTracker/issues/170
 								instManLocked->setPitchSSGType(idx, SequenceType::AbsoluteSequence);
 								break;
 							}
@@ -1495,11 +1564,12 @@ AbstractInstrument* BtiIO::load(const BinaryContainer& ctr, const std::string& f
 					csr += 2;
 					for (uint16_t l = 0; l < seqLen; ++l) {
 						uint16_t data = ctr.readUint16(csr);
-						csr += 6;	// Skip subdata
+						csr += 2;
 						if (l == 0)
 							instManLocked->setEnvelopeADPCMSequenceData(idx, 0, data);
 						else
 							instManLocked->addEnvelopeADPCMSequenceData(idx, data);
+						if (fileVersion < Version::toBCD(1, 5, 0)) csr += 4;
 					}
 
 					uint16_t loopCnt = ctr.readUint16(csr);
@@ -1518,7 +1588,7 @@ AbstractInstrument* BtiIO::load(const BinaryContainer& ctr, const std::string& f
 						instManLocked->setEnvelopeADPCMRelease(idx, InstrumentSequenceRelease(InstrumentSequenceRelease::NoRelease));
 						break;
 						// Release point check (prevents a bug)
-						// https://github.com/rerrahkr/BambooTracker/issues/11
+						// https://github.com/BambooTracker/BambooTracker/issues/11
 					case 0x01:	// Fixed
 					{
 						uint16_t pos = ctr.readUint16(csr);
@@ -1593,7 +1663,7 @@ AbstractInstrument* BtiIO::load(const BinaryContainer& ctr, const std::string& f
 						uint16_t pos = ctr.readUint16(csr);
 						csr += 2;
 						// Release point check (prevents a bug)
-						// https://github.com/rerrahkr/BambooTracker/issues/11
+						// https://github.com/BambooTracker/BambooTracker/issues/11
 						if (pos < seqLen) instManLocked->setArpeggioADPCMRelease(idx, InstrumentSequenceRelease(InstrumentSequenceRelease::FixedRelease, pos));
 						else instManLocked->setArpeggioADPCMRelease(idx, InstrumentSequenceRelease(InstrumentSequenceRelease::NoRelease));
 						break;
@@ -1660,7 +1730,7 @@ AbstractInstrument* BtiIO::load(const BinaryContainer& ctr, const std::string& f
 						uint16_t pos = ctr.readUint16(csr);
 						csr += 2;
 						// Release point check (prevents a bug)
-						// https://github.com/rerrahkr/BambooTracker/issues/11
+						// https://github.com/BambooTracker/BambooTracker/issues/11
 						if (pos < seqLen) instManLocked->setPitchADPCMRelease(idx, InstrumentSequenceRelease(InstrumentSequenceRelease::FixedRelease, pos));
 						else instManLocked->setPitchADPCMRelease(idx, InstrumentSequenceRelease(InstrumentSequenceRelease::NoRelease));
 						break;
@@ -1683,6 +1753,58 @@ AbstractInstrument* BtiIO::load(const BinaryContainer& ctr, const std::string& f
 					instPropCsr += ofs;
 					break;
 				}
+				case 0x44:	// ADPCM pan
+				{
+					int idx = *numIt++;
+					auto adpcm = dynamic_cast<InstrumentADPCM*>(inst);
+					adpcm->setPanEnabled(true);
+					adpcm->setPanNumber(idx);
+					uint16_t ofs = ctr.readUint16(instPropCsr);
+					size_t csr = instPropCsr + 2;
+
+					uint16_t seqLen = ctr.readUint16(csr);
+					csr += 2;
+					for (uint16_t l = 0; l < seqLen; ++l) {
+						uint16_t data = ctr.readUint16(csr);
+						csr += 2;
+						if (l == 0)
+							instManLocked->setPanADPCMSequenceData(idx, 0, data);
+						else
+							instManLocked->addPanADPCMSequenceData(idx, data);
+					}
+
+					uint16_t loopCnt = ctr.readUint16(csr);
+					csr += 2;
+					for (uint16_t l = 0; l < loopCnt; ++l) {
+						int begin = ctr.readUint16(csr);
+						csr += 2;
+						int end = ctr.readUint16(csr);
+						csr += 2;
+						int times = ctr.readUint8(csr++);
+						instManLocked->addPanADPCMLoop(idx, InstrumentSequenceLoop(begin, end, times));
+					}
+
+					switch (ctr.readUint8(csr++)) {
+					case 0x00:	// No release
+						instManLocked->setPanADPCMRelease(idx, InstrumentSequenceRelease(InstrumentSequenceRelease::NoRelease));
+						break;
+					case 0x01:	// Fixed
+					{
+						uint16_t pos = ctr.readUint16(csr);
+						csr += 2;
+						// Release point check (prevents a bug)
+						// https://github.com/BambooTracker/BambooTracker/issues/11
+						if (pos < seqLen) instManLocked->setPanADPCMRelease(idx, InstrumentSequenceRelease(InstrumentSequenceRelease::FixedRelease, pos));
+						else instManLocked->setPanADPCMRelease(idx, InstrumentSequenceRelease(InstrumentSequenceRelease::NoRelease));
+						break;
+					}
+					default:
+						throw FileCorruptionError(FileType::Inst, csr);
+					}
+
+					instPropCsr += ofs;
+					break;
+				}
 				default:
 					throw FileCorruptionError(FileType::Inst, instPropCsr);
 				}
@@ -1815,7 +1937,7 @@ void BtiIO::save(BinaryContainer& ctr,
 			ctr.appendUint8(0x03);
 			auto kit = std::dynamic_pointer_cast<InstrumentDrumkit>(inst);
 			std::vector<int> keys = kit->getAssignedKeys();
-			ctr.appendUint8(keys.size());
+			ctr.appendUint8(static_cast<uint8_t>(keys.size()));
 			int sampCnt = 0;
 			std::unordered_map<int, int> sampMap;
 			for (const int& key : keys) {
@@ -1823,7 +1945,8 @@ void BtiIO::save(BinaryContainer& ctr,
 				int samp = kit->getSampleNumber(key);
 				if (!sampMap.count(samp)) sampMap[samp] = sampCnt++;
 				ctr.appendUint8(static_cast<uint8_t>(sampMap[samp]));
-				ctr.appendInt8(kit->getPitch(key));
+				ctr.appendInt8(static_cast<int8_t>(kit->getPitch(key)));
+				ctr.appendUint8(static_cast<uint8_t>(kit->getPan(key)));
 			}
 			break;
 		}
@@ -2024,6 +2147,45 @@ void BtiIO::save(BinaryContainer& ctr,
 			}
 			ctr.writeUint16(ofs, static_cast<uint16_t>(ctr.size() - ofs));
 		}
+
+		// FM pan
+		if (instFM->getPanEnabled()) {
+			int panNum = instFM->getPanNumber();
+			ctr.appendUint8(0x2a);
+			size_t ofs = ctr.size();
+			ctr.appendUint16(0);	// Dummy offset
+			auto seq = instManLocked->getPanFMSequence(panNum);
+			ctr.appendUint16(static_cast<uint16_t>(seq.size()));
+			for (auto& unit : seq) {
+				ctr.appendUint16(static_cast<uint16_t>(unit.data));
+			}
+			auto loops = instManLocked->getPanFMLoopRoot(panNum).getAllLoops();
+			ctr.appendUint16(static_cast<uint16_t>(loops.size()));
+			for (auto& loop : loops) {
+				ctr.appendUint16(static_cast<uint16_t>(loop.getBeginPos()));
+				ctr.appendUint16(static_cast<uint16_t>(loop.getEndPos()));
+				ctr.appendUint8(static_cast<uint8_t>(loop.getTimes()));
+			}
+			auto release = instManLocked->getPanFMRelease(panNum);
+			switch (release.getType()) {
+			case InstrumentSequenceRelease::NoRelease:
+				ctr.appendUint8(0x00);
+				break;
+			case InstrumentSequenceRelease::FixedRelease:
+				ctr.appendUint8(0x01);
+				ctr.appendUint16(static_cast<uint16_t>(release.getBeginPos()));
+				break;
+			case InstrumentSequenceRelease::AbsoluteRelease:
+				ctr.appendUint8(0x02);
+				ctr.appendUint16(static_cast<uint16_t>(release.getBeginPos()));
+				break;
+			case InstrumentSequenceRelease::RelativeRelease:
+				ctr.appendUint8(0x03);
+				ctr.appendUint16(static_cast<uint16_t>(release.getBeginPos()));
+				break;
+			}
+			ctr.writeUint16(ofs, static_cast<uint16_t>(ctr.size() - ofs));
+		}
 		break;
 	}
 	case InstrumentType::SSG:
@@ -2271,7 +2433,6 @@ void BtiIO::save(BinaryContainer& ctr,
 			ctr.appendUint16(static_cast<uint16_t>(seq.size()));
 			for (auto& unit : seq) {
 				ctr.appendUint16(static_cast<uint16_t>(unit.data));
-				ctr.appendInt32(0);	// Dummy set　for past format
 			}
 			auto loops = instManLocked->getEnvelopeADPCMLoopRoot(envNum).getAllLoops();
 			ctr.appendUint16(static_cast<uint16_t>(loops.size()));
@@ -2390,6 +2551,45 @@ void BtiIO::save(BinaryContainer& ctr,
 			}
 			ctr.writeUint16(ofs, static_cast<uint16_t>(ctr.size() - ofs));
 		}
+
+		// ADPCM pan
+		if (instADPCM->getPanEnabled()) {
+			int panNum = instADPCM->getPanNumber();
+			ctr.appendUint8(0x44);
+			size_t ofs = ctr.size();
+			ctr.appendUint16(0);	// Dummy offset
+			auto seq = instManLocked->getPanADPCMSequence(panNum);
+			ctr.appendUint16(static_cast<uint16_t>(seq.size()));
+			for (auto& unit : seq) {
+				ctr.appendUint16(static_cast<uint16_t>(unit.data));
+			}
+			auto loops = instManLocked->getPanADPCMLoopRoot(panNum).getAllLoops();
+			ctr.appendUint16(static_cast<uint16_t>(loops.size()));
+			for (auto& loop : loops) {
+				ctr.appendUint16(static_cast<uint16_t>(loop.getBeginPos()));
+				ctr.appendUint16(static_cast<uint16_t>(loop.getEndPos()));
+				ctr.appendUint8(static_cast<uint8_t>(loop.getTimes()));
+			}
+			auto release = instManLocked->getPanADPCMRelease(panNum);
+			switch (release.getType()) {
+			case InstrumentSequenceRelease::NoRelease:
+				ctr.appendUint8(0x00);
+				break;
+			case InstrumentSequenceRelease::FixedRelease:
+				ctr.appendUint8(0x01);
+				ctr.appendUint16(static_cast<uint16_t>(release.getBeginPos()));
+				break;
+			case InstrumentSequenceRelease::AbsoluteRelease:
+				ctr.appendUint8(0x02);
+				ctr.appendUint16(static_cast<uint16_t>(release.getBeginPos()));
+				break;
+			case InstrumentSequenceRelease::RelativeRelease:
+				ctr.appendUint8(0x03);
+				ctr.appendUint16(static_cast<uint16_t>(release.getBeginPos()));
+				break;
+			}
+			ctr.writeUint16(ofs, static_cast<uint16_t>(ctr.size() - ofs));
+		}
 		break;
 	}
 	case InstrumentType::Drumkit:
