@@ -158,6 +158,12 @@ size_t loadInstrumentSection(std::weak_ptr<InstrumentsManager> instMan,
 					iCsr += 1;
 				}
 			}
+			tmp = ctr.readUint8(iCsr);
+			if (version >= Version::toBCD(1, 6, 0)) {
+				instFM->setPanEnabled((0x80 & tmp) ? false : true);
+				instFM->setPanNumber(0x7f & tmp);
+				/* iCsr += 1; */
+			}
 			instManLocked->addInstrument(instFM);
 			break;
 		}
@@ -202,7 +208,13 @@ size_t loadInstrumentSection(std::weak_ptr<InstrumentsManager> instMan,
 			tmp = ctr.readUint8(iCsr);
 			instADPCM->setPitchEnabled((0x80 & tmp) ? false : true);
 			instADPCM->setPitchNumber(0x7f & tmp);
-			/* iCsr += 1; */
+			iCsr += 1;
+			if (version >= Version::toBCD(1, 6, 0)) {
+				tmp = ctr.readUint8(iCsr);
+				instADPCM->setPanEnabled((0x80 & tmp) ? false : true);
+				instADPCM->setPanNumber(0x7f & tmp);
+				/* iCsr += 1; */
+			}
 			instManLocked->addInstrument(instADPCM);
 			break;
 		}
@@ -215,6 +227,9 @@ size_t loadInstrumentSection(std::weak_ptr<InstrumentsManager> instMan,
 				instKit->setSampleEnabled(key, true);
 				instKit->setSampleNumber(key, ctr.readUint8(iCsr++));
 				instKit->setPitch(key, ctr.readInt8(iCsr++));
+				if (version >= Version::toBCD(1, 6, 0)) {
+					instKit->setPan(key, ctr.readUint8(iCsr++));
+				}
 			}
 			instManLocked->addInstrument(instKit);
 			break;
@@ -267,7 +282,7 @@ size_t loadOperatorSequence(FMEnvelopeParameter param, size_t instMemCsr,
 	{
 		uint16_t pos = ctr.readUint16(csr);
 		csr += 2;
-		// Release point check (prevents a bug; see rerrahkr/BambooTracker issue #11)
+		// Release point check (prevents a bug; see BambooTracker/BambooTracker issue #11)
 		if (pos < seqLen) instManLocked->setOperatorSequenceFMRelease(param, idx, InstrumentSequenceRelease(InstrumentSequenceRelease::FixedRelease, pos));
 		else instManLocked->setOperatorSequenceFMRelease(param, idx, InstrumentSequenceRelease(InstrumentSequenceRelease::NoRelease));
 		break;
@@ -693,7 +708,7 @@ size_t loadInstrumentPropertySection(std::weak_ptr<InstrumentsManager> instMan,
 					uint16_t pos = ctr.readUint16(csr);
 					csr += 2;
 					// Release point check (prevents a bug)
-					// https://github.com/rerrahkr/BambooTracker/issues/11
+					// https://github.com/BambooTracker/BambooTracker/issues/11
 					if (pos < seqLen) instManLocked->setArpeggioFMRelease(idx, InstrumentSequenceRelease(InstrumentSequenceRelease::FixedRelease, pos));
 					else instManLocked->setArpeggioFMRelease(idx, InstrumentSequenceRelease(InstrumentSequenceRelease::NoRelease));
 					break;
@@ -715,7 +730,7 @@ size_t loadInstrumentPropertySection(std::weak_ptr<InstrumentsManager> instMan,
 					default:
 						if (version < Version::toBCD(1, 3, 2)) {
 							// Recover deep clone bug
-							// https://github.com/rerrahkr/BambooTracker/issues/170
+							// https://github.com/BambooTracker/BambooTracker/issues/170
 							instManLocked->setArpeggioFMType(idx, SequenceType::AbsoluteSequence);
 							break;
 						}
@@ -769,7 +784,7 @@ size_t loadInstrumentPropertySection(std::weak_ptr<InstrumentsManager> instMan,
 					uint16_t pos = ctr.readUint16(csr);
 					csr += 2;
 					// Release point check (prevents a bug)
-					// https://github.com/rerrahkr/BambooTracker/issues/11
+					// https://github.com/BambooTracker/BambooTracker/issues/11
 					if (pos < seqLen) instManLocked->setPitchFMRelease(idx, InstrumentSequenceRelease(InstrumentSequenceRelease::FixedRelease, pos));
 					else instManLocked->setPitchFMRelease(idx, InstrumentSequenceRelease(InstrumentSequenceRelease::NoRelease));
 					break;
@@ -788,7 +803,7 @@ size_t loadInstrumentPropertySection(std::weak_ptr<InstrumentsManager> instMan,
 					default:
 						if (version < Version::toBCD(1, 3, 2)) {
 							// Recover deep clone bug
-							// https://github.com/rerrahkr/BambooTracker/issues/170
+							// https://github.com/BambooTracker/BambooTracker/issues/170
 							instManLocked->setPitchFMType(idx, SequenceType::AbsoluteSequence);
 							break;
 						}
@@ -802,6 +817,58 @@ size_t loadInstrumentPropertySection(std::weak_ptr<InstrumentsManager> instMan,
 			}
 			break;
 		}
+		case 0x2a:	// FM pan
+		{
+			uint8_t cnt = ctr.readUint8(instPropCsr++);
+			for (size_t i = 0; i < cnt; ++i) {
+				uint8_t idx = ctr.readUint8(instPropCsr++);
+				uint16_t ofs = ctr.readUint16(instPropCsr);
+				size_t csr = instPropCsr + 2;
+
+				uint16_t seqLen = ctr.readUint16(csr);
+				csr += 2;
+				for (uint16_t l = 0; l < seqLen; ++l) {
+					uint16_t data = ctr.readUint16(csr);
+					csr += 2;
+					if (l == 0)
+						instManLocked->setPanFMSequenceData(idx, 0, data);
+					else
+						instManLocked->addPanFMSequenceData(idx, data);
+				}
+
+				uint16_t loopCnt = ctr.readUint16(csr);
+				csr += 2;
+				for (uint16_t l = 0; l < loopCnt; ++l) {
+					int begin = ctr.readUint16(csr);
+					csr += 2;
+					int end = ctr.readUint16(csr);
+					csr += 2;
+					int times = ctr.readUint8(csr++);
+					instManLocked->addPanFMLoop(idx, InstrumentSequenceLoop(begin, end, times));
+				}
+
+				switch (ctr.readUint8(csr++)) {
+				case 0x00:	// No release
+					instManLocked->setPanFMRelease(idx, InstrumentSequenceRelease(InstrumentSequenceRelease::NoRelease));
+					break;
+				case 0x01:	// Fixed
+				{
+					uint16_t pos = ctr.readUint16(csr);
+					csr += 2;
+					// Release point check (prevents a bug)
+					// https://github.com/BambooTracker/BambooTracker/issues/11
+					if (pos < seqLen) instManLocked->setPanFMRelease(idx, InstrumentSequenceRelease(InstrumentSequenceRelease::FixedRelease, pos));
+					else instManLocked->setPanFMRelease(idx, InstrumentSequenceRelease(InstrumentSequenceRelease::NoRelease));
+					break;
+				}
+				default:
+					throw FileCorruptionError(FileType::Mod, csr);
+				}
+
+				instPropCsr += ofs;
+			}
+			break;
+		}
 		case 0x30:	// SSG waveform
 		{
 			uint8_t cnt = ctr.readUint8(instPropCsr++);
@@ -867,7 +934,7 @@ size_t loadInstrumentPropertySection(std::weak_ptr<InstrumentsManager> instMan,
 					uint16_t pos = ctr.readUint16(csr);
 					csr += 2;
 					// Release point check (prevents a bug)
-					// https://github.com/rerrahkr/BambooTracker/issues/11
+					// https://github.com/BambooTracker/BambooTracker/issues/11
 					if (pos < seqLen) instManLocked->setWaveformSSGRelease(idx, InstrumentSequenceRelease(InstrumentSequenceRelease::FixedRelease, pos));
 					else instManLocked->setWaveformSSGRelease(idx, InstrumentSequenceRelease(InstrumentSequenceRelease::NoRelease));
 					break;
@@ -929,7 +996,7 @@ size_t loadInstrumentPropertySection(std::weak_ptr<InstrumentsManager> instMan,
 					uint16_t pos = ctr.readUint16(csr);
 					csr += 2;
 					// Release point check (prevents a bug)
-					// https://github.com/rerrahkr/BambooTracker/issues/11
+					// https://github.com/BambooTracker/BambooTracker/issues/11
 					if (pos < seqLen) instManLocked->setToneNoiseSSGRelease(idx, InstrumentSequenceRelease(InstrumentSequenceRelease::FixedRelease, pos));
 					else instManLocked->setToneNoiseSSGRelease(idx, InstrumentSequenceRelease(InstrumentSequenceRelease::NoRelease));
 					break;
@@ -991,7 +1058,7 @@ size_t loadInstrumentPropertySection(std::weak_ptr<InstrumentsManager> instMan,
 					instManLocked->setEnvelopeSSGRelease(idx, InstrumentSequenceRelease(InstrumentSequenceRelease::NoRelease));
 					break;
 					// Release point check (prevents a bug)
-					// https://github.com/rerrahkr/BambooTracker/issues/11
+					// https://github.com/BambooTracker/BambooTracker/issues/11
 				case 0x01:	// Fixed
 				{
 					uint16_t pos = ctr.readUint16(csr);
@@ -1067,7 +1134,7 @@ size_t loadInstrumentPropertySection(std::weak_ptr<InstrumentsManager> instMan,
 					uint16_t pos = ctr.readUint16(csr);
 					csr += 2;
 					// Release point check (prevents a bug)
-					// https://github.com/rerrahkr/BambooTracker/issues/11
+					// https://github.com/BambooTracker/BambooTracker/issues/11
 					if (pos < seqLen) instManLocked->setArpeggioSSGRelease(idx, InstrumentSequenceRelease(InstrumentSequenceRelease::FixedRelease, pos));
 					else instManLocked->setArpeggioSSGRelease(idx, InstrumentSequenceRelease(InstrumentSequenceRelease::NoRelease));
 					break;
@@ -1089,7 +1156,7 @@ size_t loadInstrumentPropertySection(std::weak_ptr<InstrumentsManager> instMan,
 					default:
 						if (version < Version::toBCD(1, 3, 2)) {
 							// Recover deep clone bug
-							// https://github.com/rerrahkr/BambooTracker/issues/170
+							// https://github.com/BambooTracker/BambooTracker/issues/170
 							instManLocked->setArpeggioSSGType(idx, SequenceType::AbsoluteSequence);
 							break;
 						}
@@ -1143,7 +1210,7 @@ size_t loadInstrumentPropertySection(std::weak_ptr<InstrumentsManager> instMan,
 					uint16_t pos = ctr.readUint16(csr);
 					csr += 2;
 					// Release point check (prevents a bug)
-					// https://github.com/rerrahkr/BambooTracker/issues/11
+					// https://github.com/BambooTracker/BambooTracker/issues/11
 					if (pos < seqLen) instManLocked->setPitchSSGRelease(idx, InstrumentSequenceRelease(InstrumentSequenceRelease::FixedRelease, pos));
 					else instManLocked->setPitchSSGRelease(idx, InstrumentSequenceRelease(InstrumentSequenceRelease::NoRelease));
 					break;
@@ -1162,7 +1229,7 @@ size_t loadInstrumentPropertySection(std::weak_ptr<InstrumentsManager> instMan,
 					default:
 						if (version < Version::toBCD(1, 3, 2)) {
 							// Recover deep clone bug
-							// https://github.com/rerrahkr/BambooTracker/issues/170
+							// https://github.com/BambooTracker/BambooTracker/issues/170
 							instManLocked->setPitchSSGType(idx, SequenceType::AbsoluteSequence);
 							break;
 						}
@@ -1210,11 +1277,12 @@ size_t loadInstrumentPropertySection(std::weak_ptr<InstrumentsManager> instMan,
 				csr += 2;
 				for (uint16_t l = 0; l < seqLen; ++l) {
 					uint16_t data = ctr.readUint16(csr);
-					csr += 6;	// Skip subdata
+					csr += 2;
 					if (l == 0)
 						instManLocked->setEnvelopeADPCMSequenceData(idx, 0, data);
 					else
 						instManLocked->addEnvelopeADPCMSequenceData(idx, data);
+					if (version < Version::toBCD(1, 6, 0)) csr += 4;
 				}
 
 				uint16_t loopCnt = ctr.readUint16(csr);
@@ -1233,7 +1301,7 @@ size_t loadInstrumentPropertySection(std::weak_ptr<InstrumentsManager> instMan,
 					instManLocked->setEnvelopeADPCMRelease(idx, InstrumentSequenceRelease(InstrumentSequenceRelease::NoRelease));
 					break;
 					// Release point check (prevents a bug)
-					// https://github.com/rerrahkr/BambooTracker/issues/11
+					// https://github.com/BambooTracker/BambooTracker/issues/11
 				case 0x01:	// Fixed
 				{
 					uint16_t pos = ctr.readUint16(csr);
@@ -1306,7 +1374,7 @@ size_t loadInstrumentPropertySection(std::weak_ptr<InstrumentsManager> instMan,
 					uint16_t pos = ctr.readUint16(csr);
 					csr += 2;
 					// Release point check (prevents a bug)
-					// https://github.com/rerrahkr/BambooTracker/issues/11
+					// https://github.com/BambooTracker/BambooTracker/issues/11
 					if (pos < seqLen) instManLocked->setArpeggioADPCMRelease(idx, InstrumentSequenceRelease(InstrumentSequenceRelease::FixedRelease, pos));
 					else instManLocked->setArpeggioADPCMRelease(idx, InstrumentSequenceRelease(InstrumentSequenceRelease::NoRelease));
 					break;
@@ -1371,7 +1439,7 @@ size_t loadInstrumentPropertySection(std::weak_ptr<InstrumentsManager> instMan,
 					uint16_t pos = ctr.readUint16(csr);
 					csr += 2;
 					// Release point check (prevents a bug)
-					// https://github.com/rerrahkr/BambooTracker/issues/11
+					// https://github.com/BambooTracker/BambooTracker/issues/11
 					if (pos < seqLen) instManLocked->setPitchADPCMRelease(idx, InstrumentSequenceRelease(InstrumentSequenceRelease::FixedRelease, pos));
 					else instManLocked->setPitchADPCMRelease(idx, InstrumentSequenceRelease(InstrumentSequenceRelease::NoRelease));
 					break;
@@ -1394,6 +1462,58 @@ size_t loadInstrumentPropertySection(std::weak_ptr<InstrumentsManager> instMan,
 			}
 			break;
 		}
+		case 0x44:	// ADPCM pan
+		{
+			uint8_t cnt = ctr.readUint8(instPropCsr++);
+			for (size_t i = 0; i < cnt; ++i) {
+				uint8_t idx = ctr.readUint8(instPropCsr++);
+				uint16_t ofs = ctr.readUint16(instPropCsr);
+				size_t csr = instPropCsr + 2;
+
+				uint16_t seqLen = ctr.readUint16(csr);
+				csr += 2;
+				for (uint16_t l = 0; l < seqLen; ++l) {
+					uint16_t data = ctr.readUint16(csr);
+					csr += 2;
+					if (l == 0)
+						instManLocked->setPanADPCMSequenceData(idx, 0, data);
+					else
+						instManLocked->addPanADPCMSequenceData(idx, data);
+				}
+
+				uint16_t loopCnt = ctr.readUint16(csr);
+				csr += 2;
+				for (uint16_t l = 0; l < loopCnt; ++l) {
+					int begin = ctr.readUint16(csr);
+					csr += 2;
+					int end = ctr.readUint16(csr);
+					csr += 2;
+					int times = ctr.readUint8(csr++);
+					instManLocked->addPanADPCMLoop(idx, InstrumentSequenceLoop(begin, end, times));
+				}
+
+				switch (ctr.readUint8(csr++)) {
+				case 0x00:	// No release
+					instManLocked->setPanADPCMRelease(idx, InstrumentSequenceRelease(InstrumentSequenceRelease::NoRelease));
+					break;
+				case 0x01:	// Fixed
+				{
+					uint16_t pos = ctr.readUint16(csr);
+					csr += 2;
+					// Release point check (prevents a bug)
+					// https://github.com/BambooTracker/BambooTracker/issues/11
+					if (pos < seqLen) instManLocked->setPanADPCMRelease(idx, InstrumentSequenceRelease(InstrumentSequenceRelease::FixedRelease, pos));
+					else instManLocked->setPanADPCMRelease(idx, InstrumentSequenceRelease(InstrumentSequenceRelease::NoRelease));
+					break;
+				}
+				default:
+					throw FileCorruptionError(FileType::Mod, csr);
+				}
+
+				instPropCsr += ofs;
+			}
+			break;
+		}
 		default:
 			throw FileCorruptionError(FileType::Mod, instPropCsr);
 		}
@@ -1466,6 +1586,13 @@ size_t loadSongSection(std::weak_ptr<Module> mod, const BinaryContainer& ctr,
 						   static_cast<int>(tempo), groove, static_cast<int>(speed), ptnSize);
 		auto& song = modLocked->getSong(idx);
 
+		if (version >= Version::toBCD(1, 6, 0)) {
+			const uint8_t invisCnt = ctr.readUint8(scsr++);
+			for (uint8_t invis = 0; invis < invisCnt; ++invis) {
+				song.getTrack(ctr.readUint8(scsr++)).setVisibility(false);
+			}
+		}
+
 		// Bookmark
 		if (Version::toBCD(1, 4, 1) <= version) {
 			int bmSize = ctr.readUint8(scsr++);
@@ -1480,6 +1607,17 @@ size_t loadSongSection(std::weak_ptr<Module> mod, const BinaryContainer& ctr,
 			}
 		}
 
+		// Bookmark
+		if (Version::toBCD(1, 6, 0) <= version) {
+			int sgnSize = ctr.readUint8(scsr++);
+			for (int i = 0; i < sgnSize; ++i) {
+				auto type = static_cast<KeySignature::Type>(ctr.readUint8(scsr++));
+				int order = ctr.readUint8(scsr++);
+				int step = ctr.readUint8(scsr++);
+				song.addKeySignature(type, order, step);
+			}
+		}
+
 		while (scsr < songCsr) {
 			// Track
 			uint8_t trackIdx = ctr.readUint8(scsr++);
@@ -1726,6 +1864,8 @@ void BtmIO::save(BinaryContainer& ctr, const std::weak_ptr<Module> mod,
 					tmp = static_cast<uint8_t>(instFM->getPitchNumber(type));
 					ctr.appendUint8(instFM->getPitchEnabled(type) ? tmp : (0x80 | tmp));
 				}
+				tmp = static_cast<uint8_t>(instFM->getPanNumber());
+				ctr.appendUint8(instFM->getPanEnabled() ? tmp : (0x80 | tmp));
 				break;
 			}
 			case InstrumentType::SSG:
@@ -1755,6 +1895,8 @@ void BtmIO::save(BinaryContainer& ctr, const std::weak_ptr<Module> mod,
 				ctr.appendUint8(instADPCM->getArpeggioEnabled() ? tmp : (0x80 | tmp));
 				tmp = static_cast<uint8_t>(instADPCM->getPitchNumber());
 				ctr.appendUint8(instADPCM->getPitchEnabled() ? tmp : (0x80 | tmp));
+				tmp = static_cast<uint8_t>(instADPCM->getPanNumber());
+				ctr.appendUint8(instADPCM->getPanEnabled() ? tmp : (0x80 | tmp));
 				break;
 			}
 			case InstrumentType::Drumkit:
@@ -1762,11 +1904,12 @@ void BtmIO::save(BinaryContainer& ctr, const std::weak_ptr<Module> mod,
 				ctr.appendUint8(0x03);
 				auto instKit = std::dynamic_pointer_cast<InstrumentDrumkit>(inst);
 				std::vector<int> keys = instKit->getAssignedKeys();
-				ctr.appendUint8(keys.size());
+				ctr.appendUint8(static_cast<uint8_t>(keys.size()));
 				for (const int& key : keys) {
 					ctr.appendUint8(static_cast<uint8_t>(key));
 					ctr.appendUint8(static_cast<uint8_t>(instKit->getSampleNumber(key)));
-					ctr.appendInt8(instKit->getPitch(key));
+					ctr.appendInt8(static_cast<int8_t>(instKit->getPitch(key)));
+					ctr.appendUint8(static_cast<uint8_t>(instKit->getPan(key)));
 				}
 				break;
 			}
@@ -1989,6 +2132,50 @@ void BtmIO::save(BinaryContainer& ctr, const std::weak_ptr<Module> mod,
 		}
 	}
 
+	// FM pan
+	std::vector<int> panFMIdcs = instManLocked->getPanFMEntriedIndices();
+	if (!panFMIdcs.empty()) {
+		ctr.appendUint8(0x2a);
+		ctr.appendUint8(static_cast<uint8_t>(panFMIdcs.size()));
+		for (auto& idx : panFMIdcs) {
+			ctr.appendUint8(static_cast<uint8_t>(idx));
+			size_t ofs = ctr.size();
+			ctr.appendUint16(0);	// Dummy offset
+			auto seq = instManLocked->getPanFMSequence(idx);
+			ctr.appendUint16(static_cast<uint16_t>(seq.size()));
+			for (auto& unit : seq) {
+				ctr.appendUint16(static_cast<uint16_t>(unit.data));
+			}
+			auto loops = instManLocked->getPanFMLoopRoot(idx).getAllLoops();
+			ctr.appendUint16(static_cast<uint16_t>(loops.size()));
+			for (auto& loop : loops) {
+				ctr.appendUint16(static_cast<uint16_t>(loop.getBeginPos()));
+				ctr.appendUint16(static_cast<uint16_t>(loop.getEndPos()));
+				ctr.appendUint8(static_cast<uint8_t>(loop.getTimes()));
+			}
+			auto release = instManLocked->getPanFMRelease(idx);
+			switch (release.getType()) {
+			case InstrumentSequenceRelease::NoRelease:
+				ctr.appendUint8(0x00);
+				// If release.type is NO_RELEASE, then release.begin == -1 so omit to save it.
+				break;
+			case InstrumentSequenceRelease::FixedRelease:
+				ctr.appendUint8(0x01);
+				ctr.appendUint16(static_cast<uint16_t>(release.getBeginPos()));
+				break;
+			case InstrumentSequenceRelease::AbsoluteRelease:
+				ctr.appendUint8(0x02);
+				ctr.appendUint16(static_cast<uint16_t>(release.getBeginPos()));
+				break;
+			case InstrumentSequenceRelease::RelativeRelease:
+				ctr.appendUint8(0x03);
+				ctr.appendUint16(static_cast<uint16_t>(release.getBeginPos()));
+				break;
+			}
+			ctr.writeUint16(ofs, static_cast<uint16_t>(ctr.size() - ofs));
+		}
+	}
+
 	// SSG waveform
 	std::vector<int> wfSSGIdcs = instManLocked->getWaveformSSGEntriedIndices();
 	if (!wfSSGIdcs.empty()) {
@@ -2258,7 +2445,6 @@ void BtmIO::save(BinaryContainer& ctr, const std::weak_ptr<Module> mod,
 			ctr.appendUint16(static_cast<uint16_t>(seq.size()));
 			for (auto& unit : seq) {
 				ctr.appendUint16(static_cast<uint16_t>(unit.data));
-				ctr.appendInt32(0);	// Dummy set　for past format
 			}
 			auto loops = instManLocked->getEnvelopeADPCMLoopRoot(idx).getAllLoops();
 			ctr.appendUint16(static_cast<uint16_t>(loops.size()));
@@ -2391,6 +2577,50 @@ void BtmIO::save(BinaryContainer& ctr, const std::weak_ptr<Module> mod,
 		}
 	}
 
+	// ADPCM pan
+	std::vector<int> panADPCMIdcs = instManLocked->getPanADPCMEntriedIndices();
+	if (!panADPCMIdcs.empty()) {
+		ctr.appendUint8(0x44);
+		ctr.appendUint8(static_cast<uint8_t>(panADPCMIdcs.size()));
+		for (auto& idx : panADPCMIdcs) {
+			ctr.appendUint8(static_cast<uint8_t>(idx));
+			size_t ofs = ctr.size();
+			ctr.appendUint16(0);	// Dummy offset
+			auto seq = instManLocked->getPanADPCMSequence(idx);
+			ctr.appendUint16(static_cast<uint16_t>(seq.size()));
+			for (auto& unit : seq) {
+				ctr.appendUint16(static_cast<uint16_t>(unit.data));
+			}
+			auto loops = instManLocked->getPanADPCMLoopRoot(idx).getAllLoops();
+			ctr.appendUint16(static_cast<uint16_t>(loops.size()));
+			for (auto& loop : loops) {
+				ctr.appendUint16(static_cast<uint16_t>(loop.getBeginPos()));
+				ctr.appendUint16(static_cast<uint16_t>(loop.getEndPos()));
+				ctr.appendUint8(static_cast<uint8_t>(loop.getTimes()));
+			}
+			auto release = instManLocked->getPanADPCMRelease(idx);
+			switch (release.getType()) {
+			case InstrumentSequenceRelease::NoRelease:
+				ctr.appendUint8(0x00);
+				// If release.type is NO_RELEASE, then release.begin == -1 so omit to save it.
+				break;
+			case InstrumentSequenceRelease::FixedRelease:
+				ctr.appendUint8(0x01);
+				ctr.appendUint16(static_cast<uint16_t>(release.getBeginPos()));
+				break;
+			case InstrumentSequenceRelease::AbsoluteRelease:
+				ctr.appendUint8(0x02);
+				ctr.appendUint16(static_cast<uint16_t>(release.getBeginPos()));
+				break;
+			case InstrumentSequenceRelease::RelativeRelease:
+				ctr.appendUint8(0x03);
+				ctr.appendUint16(static_cast<uint16_t>(release.getBeginPos()));
+				break;
+			}
+			ctr.writeUint16(ofs, static_cast<uint16_t>(ctr.size() - ofs));
+		}
+	}
+
 	ctr.writeUint32(instPropOfs, ctr.size() - instPropOfs);
 
 
@@ -2438,6 +2668,12 @@ void BtmIO::save(BinaryContainer& ctr, const std::weak_ptr<Module> mod,
 		case SongType::FM3chExpanded:	ctr.appendUint8(0x01);	break;
 		default:	throw std::out_of_range("");
 		}
+		std::vector<int> inviss;
+		for (const auto& attrib : style.trackAttribs) {
+			if (!sng.getTrack(attrib.number).isVisible()) inviss.push_back(attrib.number);
+		}
+		ctr.appendUint8(static_cast<uint8_t>(inviss.size()));
+		if (!inviss.empty()) for (int n : inviss) ctr.appendUint8(static_cast<uint8_t>(n));
 
 		// Bookmark
 		size_t bmSize = sng.getBookmarkSize();
@@ -2450,6 +2686,16 @@ void BtmIO::save(BinaryContainer& ctr, const std::weak_ptr<Module> mod,
 			ctr.appendUint8(static_cast<uint8_t>(bm.step));
 		}
 
+		// Key signature
+		size_t sgnSize = sng.getKeySignatureSize();
+		ctr.appendUint8(static_cast<uint8_t>(sgnSize));
+		for (size_t i = 0; i < sgnSize; ++i) {
+			KeySignature signature = sng.getKeySignature(static_cast<int>(i));
+			ctr.appendUint8(static_cast<uint8_t>(signature.type));
+			ctr.appendUint8(static_cast<uint8_t>(signature.order));
+			ctr.appendUint8(static_cast<uint8_t>(signature.step));
+		}
+
 		// Track
 		for (auto& attrib : style.trackAttribs) {
 			ctr.appendUint8(static_cast<uint8_t>(attrib.number));
