@@ -50,21 +50,23 @@ int main(int argc, char* argv[])
 {
 	try {
 		std::shared_ptr<Configuration> config = std::make_shared<Configuration>();
-		io::loadConfiguration(config);
+		bool hasSuccessed = io::loadConfiguration(config);
 
-#if QT_VERSION >= QT_VERSION_CHECK(5, 6, 0)
+#if QT_VERSION >= QT_VERSION_CHECK(5, 6, 0) && QT_VERSION < QT_VERSION_CHECK(6, 0, 0)
 		QApplication::setAttribute(Qt::AA_EnableHighDpiScaling);
 #endif
-
-		std::unique_ptr<QApplicationWrapper> a(std::make_unique<QApplicationWrapper>(argc, argv));
+#if QT_VERSION >= QT_VERSION_CHECK(5, 14, 0)
+		QApplication::setHighDpiScaleFactorRoundingPolicy(Qt::HighDpiScaleFactorRoundingPolicy::RoundPreferFloor);
+#endif
+		QApplicationWrapper a(argc, argv);
 		if (config->getEnableTranslation()) setupTranslations();
-		a->setWindowIcon(QIcon(":/icon/app_icon"));
+		a.setWindowIcon(QIcon(":/icon/app_icon"));
 
 		QString filePath = (argc > 1) ? argv[argc - 1] : "";	// Last argument file
 
-		std::unique_ptr<MainWindow> w(std::make_unique<MainWindow>(config, filePath));
-		w->show();
-		int ret = a->exec();
+		MainWindow w(config, filePath, !hasSuccessed);
+		w.show();
+		int ret = a.exec();
 
 		io::saveConfiguration(config);
 		if (ret) QMessageBox::critical(nullptr, QObject::tr("Error"), QObject::tr("An unknown error occurred."));
@@ -94,14 +96,16 @@ void setupTranslations()
 
 	if (!qtDir.isEmpty()) {
 		QString baseName = "qt_" + lang;
-		qtTr->load(baseName, qtDir);
-		qDebug() << "Translation" << baseName << "from" << qtDir;
+		if (qtTr->load(baseName, qtDir)) {
+			qDebug() << "Translation" << baseName << "from" << qtDir;
+		}
 	}
 
 	if (!appDir.isEmpty()) {
 		QString baseName = "bamboo_tracker_" + lang;
-		appTr->load(baseName, appDir);
-		qDebug() << "Translation" << baseName << "from" << appDir;
+		if (appTr->load(baseName, appDir)) {
+			qDebug() << "Translation" << baseName << "from" << appDir;
+		}
 	}
 
 	a->installTranslator(qtTr);
@@ -123,8 +127,12 @@ QString findQtTranslationsDir()
 	return QApplication::applicationDirPath() + "/lang";
 #else
 	// the files are located in the installation of Qt
+#if QT_VERSION >= QT_VERSION_CHECK(6, 0, 0)
+	return QLibraryInfo::path(QLibraryInfo::TranslationsPath);
+#else
 	return QLibraryInfo::location(QLibraryInfo::TranslationsPath);
 #endif
+#endif
 }
 
 // Finds the location of our translation catalogs
