@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2019-2021 Rerrah
+ * Copyright (C) 2019-2022 Rerrah
  *
  * Permission is hereby granted, free of charge, to any person
  * obtaining a copy of this software and associated documentation
@@ -172,6 +172,9 @@ EffectType validateEffectId(SoundSource src, const std::string& id)
 			return EffectType::NoEffect;
 		}
 	}
+	else if (id == "0K") {
+		return EffectType::Retrigger;
+	}
 	else if (id == "0O") {
 		return EffectType::Groove;
 	}
@@ -253,6 +256,24 @@ EffectType validateEffectId(SoundSource src, const std::string& id)
 			return EffectType::NoEffect;
 		}
 	}
+	else if (id == "EA") {
+		switch (src) {
+		case SoundSource::FM:
+		case SoundSource::SSG:
+		case SoundSource::ADPCM:
+			return EffectType::XVolumeSlide;
+		default:
+			return EffectType::NoEffect;
+		}
+	}
+	else if (id == "ES") {
+		switch (src) {
+		case SoundSource::FM:
+			return EffectType::EnvelopeReset;
+		default:
+			return EffectType::NoEffect;
+		}
+	}
 	else if (id == "FB") {
 		switch (src) {
 		case SoundSource::FM:
