@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2018-2021 Rerrah
+ * Copyright (C) 2018-2022 Rerrah
  *
  * Permission is hereby granted, free of charge, to any person
  * obtaining a copy of this software and associated documentation
@@ -58,10 +58,9 @@ public:
 	void updateRegisterStates();
 
 	// Real chip interface
-	void useSCCI(scci::SoundInterfaceManager* manager);
-	bool isUsedSCCI() const;
-	void useC86CTL(C86ctlBase* base);
-	bool isUsedC86CTL() const;
+	void connectToRealChip(RealChipInterfaceType type, RealChipInterfaceGeneratorFunc* f);
+	RealChipInterfaceType getRealChipInterfaceType() const;
+	bool hasConnectedToRealChip() const;
 
 	// Stream samples
 	void getStreamSamples(int16_t* container, size_t nSamples);
@@ -114,6 +113,7 @@ public:
 	void keyOnFM(int ch, const Note& note, bool isJam = false);
 	void keyOnFM(int ch, int echoBuf);
 	void keyOffFM(int ch, bool isJam = false);
+	void retriggerKeyOnFM(int ch, int volDiff);
 
 	// Set instrument
 	void setInstrumentFM(int ch, std::shared_ptr<InstrumentFM> inst);
@@ -136,6 +136,7 @@ public:
 	void setVibratoEffectFM(int ch, int period, int depth);
 	void setTremoloEffectFM(int ch, int period, int depth);
 	void setVolumeSlideFM(int ch, int depth, bool isUp);
+	void setXVolumeSlideFM(int ch, int factor);
 	void setDetuneFM(int ch, int pitch);
 	void setFineDetuneFM(int ch, int pitch);
 	void setNoteSlideFM(int ch, int speed, int seminote);
@@ -180,6 +181,8 @@ private:
 		std::unique_ptr<WavingEffectIterator> vibItr;
 		std::unique_ptr<WavingEffectIterator> treItr;
 		int volSld, volSldSum;
+		std::unique_ptr<XVolumeSlideEffectIterator> xVolSldItr;
+		int xvolSldSum;
 		int detune, fdetune;
 		std::unique_ptr<NoteSlideEffectIterator> nsItr;
 		int nsSum;
@@ -193,12 +196,11 @@ private:
 	std::shared_ptr<InstrumentFM> refInstFM_[6];
 	std::unique_ptr<EnvelopeFM> envFM_[6];
 	uint8_t fmOpEnables_[6];
-	/// bit0: right on/off
-	/// bit1: left on/off
-	uint8_t panFM_[6];
+	uint8_t panStateFM_[6];
+	PanIter panItrFM_[6];
 	int lfoFreq_;
 	int lfoStartCntFM_[6];
-	std::unordered_map<FMEnvelopeParameter, FMOperatorSequenceIter> opSeqItFM_[6];
+	std::unordered_map<FMEnvelopeParameter, FMOperatorSequenceIter> opSeqIrtFM_[6];
 	bool isFBCtrlFM_[6], isTLCtrlFM_[6][4], isMLCtrlFM_[6][4], isARCtrlFM_[6][4];
 	bool isDRCtrlFM_[6][4], isRRCtrlFM_[6][4];
 	bool isBrightFM_[6][4];
@@ -226,6 +228,9 @@ private:
 	void checkOperatorSequenceFM(FMChannel& fm, int type);
 	void checkVolumeEffectFM(FMChannel& fm);
 
+	void checkPanFM(FMChannel& fm, int type);
+	void writePanFM(FMChannel& fm, int pos);
+
 	void checkRealToneFMByArpeggio(FMChannel& fm);
 	void checkPortamentoFM(FMChannel& fm);
 	void checkRealToneFMByPitch(FMChannel& fm);
@@ -243,6 +248,7 @@ public:
 	void keyOnSSG(int ch, const Note& note, bool isJam = false);
 	void keyOnSSG(int ch, int echoBuf);
 	void keyOffSSG(int ch, bool isJam = false);
+	void retriggerKeyOnSSG(int ch, int volDiff);
 
 	// Set instrument
 	void setInstrumentSSG(int ch, std::shared_ptr<InstrumentSSG> inst);
@@ -259,6 +265,7 @@ public:
 	void setVibratoEffectSSG(int ch, int period, int depth);
 	void setTremoloEffectSSG(int ch, int period, int depth);
 	void setVolumeSlideSSG(int ch, int depth, bool isUp);
+	void setXVolumeSlideSSG(int ch, int factor);
 	void setDetuneSSG(int ch, int pitch);
 	void setFineDetuneSSG(int ch, int pitch);
 	void setNoteSlideSSG(int ch, int speed, int seminote);
@@ -308,6 +315,7 @@ private:
 		std::unique_ptr<WavingEffectIterator> vibItr;
 		std::unique_ptr<WavingEffectIterator> treItr;
 		int volSld, volSldSum;
+		std::unique_ptr<XVolumeSlideEffectIterator> xVolSldItr;
 		int detune, fdetune;
 		std::unique_ptr<NoteSlideEffectIterator> nsItr;
 		int nsSum;
@@ -331,6 +339,7 @@ private:
 	void writeWaveformSSGToRegister(SSGChannel& ssg);
 	void writeSquareWaveform(SSGChannel& ssg);
 
+	void xslideVolumeSsg(SSGChannel& ssg);
 	void writeEnvelopeSSGToRegister(SSGChannel& ssg);
 	void setRealVolumeSSG(SSGChannel& ssg);
 
@@ -351,6 +360,7 @@ public:
 	// Key on/off
 	void setKeyOnFlagRhythm(int ch);
 	void setKeyOffFlagRhythm(int ch);
+	void retriggerKeyOnFlagRhythm(int ch, int volDiff);
 
 	// Set volume
 	void setVolumeRhythm(int ch, int volume);
@@ -385,6 +395,7 @@ public:
 	void keyOnADPCM(const Note& note, bool isJam = false);
 	void keyOnADPCM(int echoBuf);
 	void keyOffADPCM(bool isJam = false);
+	void retriggerKeyOnADPCM(int volDiff);
 
 	// Set instrument
 	void setInstrumentADPCM(std::shared_ptr<InstrumentADPCM> inst);
@@ -406,6 +417,7 @@ public:
 	void setVibratoEffectADPCM(int period, int depth);
 	void setTremoloEffectADPCM(int period, int depth);
 	void setVolumeSlideADPCM(int depth, bool isUp);
+	void setXVolumeSlideADPCM(int factor);
 	void setDetuneADPCM(int pitch);
 	void setFineDetuneADPCM(int pitch);
 	void setNoteSlideADPCM(int speed, int seminote);
@@ -429,6 +441,7 @@ private:
 	Note baseNoteADPCM_;
 	int baseVolADPCM_, oneshotVolADPCM_;
 	uint8_t panStateADPCM_;
+	PanIter panItrADPCM_;
 	bool isMuteADPCM_;
 	bool shouldSkip1stTickExecADPCM_;	// Use to execute key on/off process in jamming
 	bool shouldWriteEnvADPCM_;
@@ -445,6 +458,7 @@ private:
 	std::unique_ptr<WavingEffectIterator> vibItrADPCM_;
 	std::unique_ptr<WavingEffectIterator> treItrADPCM_;
 	int volSldADPCM_, volSldSumADPCM_;
+	std::unique_ptr<XVolumeSlideEffectIterator> xVolSldItrAdpcm_;
 	int detuneADPCM_, fdetuneADPCM_;
 	std::unique_ptr<NoteSlideEffectIterator> nsItADPCM_;
 	int nsSumADPCM_;
@@ -461,8 +475,12 @@ private:
 	void releaseStartADPCMSequences();
 	void tickEventADPCM();
 
+	void xslideVolumeAdpcm();
 	void writeEnvelopeADPCMToRegister();
 
+	void checkPanADPCM(int type);
+	void writePanADPCM(int pos);
+
 	void checkRealToneADPCMByArpeggio();
 	void checkPortamentoADPCM();
 	void checkRealToneADPCMByPitch();
