@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2019-2021 Rerrah
+ * Copyright (C) 2019-2022 Rerrah
  *
  * Permission is hereby granted, free of charge, to any person
  * obtaining a copy of this software and associated documentation
@@ -84,7 +84,7 @@ PlaybackManager::PlaybackManager(std::shared_ptr<OPNAController> opnaCtrl,
 	songStyle_ = mod.lock()->getSong(curSongNum_).getStyle();
 
 	clearEffectMaps();
-	clearNoteDelayCounts();
+	clearDelayWithinStepCounts();
 	clearDelayBeyondStepCounts();
 }
 
@@ -97,7 +97,6 @@ void PlaybackManager::setSong(std::weak_ptr<Module> mod, int songNum)
 	/* opna mode is changed in BambooTracker class */
 
 	size_t fmch = Song::getFMChannelCount(songStyle_.type);
-	isNoteDelay_[SoundSource::FM] = std::vector<bool>(fmch);
 	effOnKeyOnMem_[SoundSource::FM] = std::vector<EffectMemory>(fmch);
 	effOnStepBeginMem_[SoundSource::FM] = std::vector<EffectMemory>(fmch);
 	directRegisterSets_[SoundSource::FM] = DirectRegisterSetSource(fmch);
@@ -107,8 +106,11 @@ void PlaybackManager::setSong(std::weak_ptr<Module> mod, int songNum)
 	volDlyValueFM_ = std::vector<int>(fmch, -1);
 	tposeDlyCntFM_ = std::vector<int>(fmch);
 	tposeDlyValueFM_ = std::vector<int>(fmch);
+	envRstDlyCntFM_ = std::vector<int>(fmch);
+	rtrgCntFM_ = std::vector<int>(fmch);
+	rtrgCntValueFM_ = std::vector<int>(fmch);
+	rtrgVolValueFM_ = std::vector<int>(fmch);
 
-	isNoteDelay_[SoundSource::SSG] = std::vector<bool>(3);
 	effOnKeyOnMem_[SoundSource::SSG] = std::vector<EffectMemory>(3);
 	effOnStepBeginMem_[SoundSource::SSG] = std::vector<EffectMemory>(3);
 	directRegisterSets_[SoundSource::SSG] = DirectRegisterSetSource(3);
@@ -118,8 +120,10 @@ void PlaybackManager::setSong(std::weak_ptr<Module> mod, int songNum)
 	volDlyValueSSG_ = std::vector<int>(3, -1);
 	tposeDlyCntSSG_ = std::vector<int>(3);
 	tposeDlyValueSSG_ = std::vector<int>(3);
+	rtrgCntSSG_ = std::vector<int>(3);
+	rtrgCntValueSSG_ = std::vector<int>(3);
+	rtrgVolValueSSG_ = std::vector<int>(3);
 
-	isNoteDelay_[SoundSource::RHYTHM] = std::vector<bool>(6);
 	effOnKeyOnMem_[SoundSource::RHYTHM] = std::vector<EffectMemory>(6);
 	effOnStepBeginMem_[SoundSource::RHYTHM] = std::vector<EffectMemory>(6);
 	directRegisterSets_[SoundSource::RHYTHM] = DirectRegisterSetSource(6);
@@ -127,8 +131,10 @@ void PlaybackManager::setSong(std::weak_ptr<Module> mod, int songNum)
 	ntCutDlyCntRhythm_ = std::vector<int>(6);
 	volDlyCntRhythm_ = std::vector<int>(6);
 	volDlyValueRhythm_ = std::vector<int>(6, -1);
+	rtrgCntRhythm_ = std::vector<int>(6);
+	rtrgCntValueRhythm_ = std::vector<int>(6);
+	rtrgVolValueRhythm_ = std::vector<int>(6);
 
-	isNoteDelay_[SoundSource::ADPCM] = std::vector<bool>(1);
 	effOnKeyOnMem_[SoundSource::ADPCM] = std::vector<EffectMemory>(1);
 	effOnStepBeginMem_[SoundSource::ADPCM] = std::vector<EffectMemory>(1);
 	directRegisterSets_[SoundSource::ADPCM] = DirectRegisterSetSource(1);
@@ -136,6 +142,9 @@ void PlaybackManager::setSong(std::weak_ptr<Module> mod, int songNum)
 	ntCutDlyCntADPCM_ = 0;
 	volDlyCntADPCM_ = 0;
 	volDlyValueADPCM_ = -1;
+	rtrgCntADPCM_ = 0;
+	rtrgCntValueADPCM_ = 0;
+	rtrgVolValueADPCM_ = 0;
 }
 
 /********** Play song **********/
@@ -201,7 +210,7 @@ void PlaybackManager::playStep(int order, int step)
 	tickCounter_.lock()->setPlayState(true);
 
 	clearEffectMaps();
-	clearNoteDelayCounts();
+	clearDelayWithinStepCounts();
 	clearDelayBeyondStepCounts();
 
 	playStateFlags_ = PlayStateFlag::PlayStep;
@@ -224,7 +233,7 @@ void PlaybackManager::startPlay()
 	tickCounter_.lock()->setPlayState(true);
 
 	clearEffectMaps();
-	clearNoteDelayCounts();
+	clearDelayWithinStepCounts();
 	clearDelayBeyondStepCounts();
 }
 
@@ -366,7 +375,7 @@ void PlaybackManager::checkValidPosition()
 /// Register update order: volume -> instrument -> effect -> key on
 void PlaybackManager::stepProcess()
 {
-	clearNoteDelayCounts();
+	clearDelayWithinStepCounts();
 	updateDelayEventCounts();
 
 	auto& song = mod_.lock()->getSong(curSongNum_);
@@ -378,29 +387,46 @@ void PlaybackManager::stepProcess()
 		size_t uch = static_cast<size_t>(attrib.channelInSource);
 		effOnKeyOnMem_[attrib.source].at(uch).clear();
 		directRegisterSets_[attrib.source].at(uch).clear();
-		using storeFunc = bool (PlaybackManager::*)(int, const Effect&);
+		using storeFunc = void (PlaybackManager::*)(int, const Effect&);
 		static const std::unordered_map<SoundSource, storeFunc> storeEffectToMap = {
 			{ SoundSource::FM, &PlaybackManager::storeEffectToMapFM },
 			{ SoundSource::SSG, &PlaybackManager::storeEffectToMapSSG },
 			{ SoundSource::RHYTHM, &PlaybackManager::storeEffectToMapRhythm },
 			{ SoundSource::ADPCM, &PlaybackManager::storeEffectToMapADPCM }
 		};
-		bool isDelay = false;
 		for (int i = 0; i < Step::N_EFFECT; ++i) {
 			Effect&& eff = effect_utils::validateEffect(attrib.source, step.getEffect(i));
-			isDelay |= (this->*storeEffectToMap.at(attrib.source))(attrib.channelInSource, eff);
+			(this->*storeEffectToMap.at(attrib.source))(attrib.channelInSource, eff);
 		}
-		isNoteDelay_[attrib.source].at(uch) = isDelay;
 	}
 
 	// Execute step events
 	bool isNextSet = executeStoredEffectsGlobal();
+	const int countsInStep = tickCounter_.lock()->getCountsInCurrentStep();
 	for (auto& attrib : songStyle_.trackAttribs) {
+		// Check whether it has been set note delay effect
+		size_t uch = static_cast<size_t>(attrib.channelInSource);
+		bool hasSetNoteDelay = false;
+		auto& mem = effOnStepBeginMem_[attrib.source].at(uch);
+		for (auto itr = mem.begin(); itr != mem.end(); ) {
+			if (itr->type == EffectType::NoteDelay) {
+				if (itr->value < countsInStep) {
+					hasSetNoteDelay = true;
+				}
+				else {
+					itr = mem.erase(itr);
+					continue;
+				}
+			}
+			++itr;
+		}
+
+		//
 		auto& step = song.getTrack(attrib.number)
 					 .getPatternFromOrderNumber(playingPos_.order).getStep(playingPos_.step);
 		switch (attrib.source) {
 		case SoundSource::FM:
-			if (isNoteDelay_[SoundSource::FM].at(attrib.channelInSource)) {
+			if (hasSetNoteDelay) {
 				// Set effect
 				executeStoredEffectsFM(attrib.channelInSource);
 				checkFMNoteDelayAndEnvelopeReset(step, attrib.channelInSource);
@@ -411,7 +437,7 @@ void PlaybackManager::stepProcess()
 			}
 			break;
 		case SoundSource::SSG:
-			if (isNoteDelay_[SoundSource::SSG].at(attrib.channelInSource)) {
+			if (hasSetNoteDelay) {
 				// Set effect
 				executeStoredEffectsSSG(attrib.channelInSource);
 				opnaCtrl_->tickEvent(SoundSource::SSG, attrib.channelInSource);
@@ -421,7 +447,7 @@ void PlaybackManager::stepProcess()
 			}
 			break;
 		case SoundSource::RHYTHM:
-			if (isNoteDelay_[SoundSource::RHYTHM].at(attrib.channelInSource)) {
+			if (hasSetNoteDelay) {
 				// Set effect
 				executeStoredEffectsRhythm(attrib.channelInSource);
 				opnaCtrl_->tickEvent(SoundSource::RHYTHM, attrib.channelInSource);
@@ -431,7 +457,7 @@ void PlaybackManager::stepProcess()
 			}
 			break;
 		case SoundSource::ADPCM:
-			if (isNoteDelay_[SoundSource::ADPCM].at(attrib.channelInSource)) {
+			if (hasSetNoteDelay) {
 				// Set effect
 				executeStoredEffectsADPCM();
 				opnaCtrl_->tickEvent(SoundSource::ADPCM, attrib.channelInSource);
@@ -474,10 +500,11 @@ void PlaybackManager::executeFMStepEvents(const Step& step, int ch, bool calledB
 	int noteNum = step.getNoteNumber();
 	switch (noteNum) {
 	case Step::NOTE_NONE:
-		if (!calledByNoteDelay) {	// When this is called by note delay, tick event will be updated in readTick
+		if (!calledByNoteDelay) {
+			// When this is called by note delay, skip delay check because it has already checked.
 			checkFMDelayEventsInTick(step, ch);
-			opnaCtrl_->tickEvent(SoundSource::FM, ch);
 		}
+		opnaCtrl_->tickEvent(SoundSource::FM, ch);
 		break;
 	case Step::NOTE_KEY_OFF:
 		opnaCtrl_->keyOffFM(ch);
@@ -524,10 +551,11 @@ void PlaybackManager::executeSSGStepEvents(const Step& step, int ch, bool called
 	int noteNum = step.getNoteNumber();
 	switch (noteNum) {
 	case Step::NOTE_NONE:
-		if (!calledByNoteDelay) {	// When this is called by note delay, tick event will be updated in readTick
+		if (!calledByNoteDelay) {
+			// When this is called by note delay, skip delay check because it has already checked.
 			checkSSGDelayEventsInTick(step, ch);
-			opnaCtrl_->tickEvent(SoundSource::SSG, ch);
 		}
+		opnaCtrl_->tickEvent(SoundSource::SSG, ch);
 		break;
 	case Step::NOTE_KEY_OFF:
 		opnaCtrl_->keyOffSSG(ch);
@@ -566,10 +594,11 @@ void PlaybackManager::executeRhythmStepEvents(const Step& step, int ch, bool cal
 	// Set key
 	switch (step.getNoteNumber()) {
 	case Step::NOTE_NONE:
-		if (!calledByNoteDelay) {	// When this is called by note delay, tick event will be updated in readTick
+		if (!calledByNoteDelay) {
+			// When this is called by note delay, skip delay check because it has already checked.
 			checkRhythmDelayEventsInTick(step, ch);
-			opnaCtrl_->tickEvent(SoundSource::RHYTHM, ch);
 		}
+		opnaCtrl_->tickEvent(SoundSource::RHYTHM, ch);
 		break;
 	case Step::NOTE_KEY_OFF:
 		opnaCtrl_->setKeyOffFlagRhythm(ch);
@@ -592,11 +621,12 @@ void PlaybackManager::executeADPCMStepEvents(const Step& step, bool calledByNote
 
 	// Set instrument
 	if (step.hasInstrument()) {
-		auto inst = instMan_.lock()->getInstrumentSharedPtr(step.getInstrumentNumber());
-		if (inst->getType() == InstrumentType::ADPCM)
-			opnaCtrl_->setInstrumentADPCM(std::dynamic_pointer_cast<InstrumentADPCM>(inst));
-		else if (inst->getType() == InstrumentType::Drumkit)
-			opnaCtrl_->setInstrumentDrumkit(std::dynamic_pointer_cast<InstrumentDrumkit>(inst));
+		if (auto inst = instMan_.lock()->getInstrumentSharedPtr(step.getInstrumentNumber())) {
+			if (inst->getType() == InstrumentType::ADPCM)
+				opnaCtrl_->setInstrumentADPCM(std::dynamic_pointer_cast<InstrumentADPCM>(inst));
+			else if (inst->getType() == InstrumentType::Drumkit)
+				opnaCtrl_->setInstrumentDrumkit(std::dynamic_pointer_cast<InstrumentDrumkit>(inst));
+		}
 	}
 
 	// Set effect
@@ -606,10 +636,11 @@ void PlaybackManager::executeADPCMStepEvents(const Step& step, bool calledByNote
 	int noteNum = step.getNoteNumber();
 	switch (noteNum) {
 	case Step::NOTE_NONE:
-		if (!calledByNoteDelay) {	// When this is called by note delay, tick event will be updated in readTick
+		if (!calledByNoteDelay) {
+			// When this is called by note delay, skip delay check because it has already checked.
 			checkADPCMDelayEventsInTick(step);
-			opnaCtrl_->tickEvent(SoundSource::ADPCM, 0);
 		}
+		opnaCtrl_->tickEvent(SoundSource::ADPCM, 0);
 		break;
 	case Step::NOTE_KEY_OFF:
 		opnaCtrl_->keyOffADPCM();
@@ -681,7 +712,7 @@ bool PlaybackManager::executeStoredEffectsGlobal()
 	return changedNextPos;
 }
 
-bool PlaybackManager::storeEffectToMapFM(int ch, const Effect& eff)
+void PlaybackManager::storeEffectToMapFM(int ch, const Effect& eff)
 {
 	switch (eff.type) {
 	case EffectType::Arpeggio:
@@ -706,26 +737,26 @@ bool PlaybackManager::storeEffectToMapFM(int ch, const Effect& eff)
 	case EffectType::DRControl:
 	case EffectType::RRControl:
 	case EffectType::Brightness:
+	case EffectType::EnvelopeReset:
+	case EffectType::Retrigger:
+	case EffectType::XVolumeSlide:
 		effOnKeyOnMem_[SoundSource::FM].at(static_cast<size_t>(ch)).enqueue(eff);
-		return false;
+		break;
 	case EffectType::SpeedTempoChange:
 	case EffectType::Groove:
 		playbackSpeedEffMem_.enqueue(eff);
-		return false;
+		break;
 	case EffectType::NoteDelay:
-		if (eff.value < tickCounter_.lock()->getSpeed()) {
-			effOnStepBeginMem_[SoundSource::FM].at(static_cast<size_t>(ch)).enqueue(eff);
-			return true;
-		}
-		return false;
+		effOnStepBeginMem_[SoundSource::FM].at(static_cast<size_t>(ch)).enqueue(eff);
+		break;
 	case EffectType::PositionJump:
 	case EffectType::SongEnd:
 	case EffectType::PatternBreak:
 		posChangeEffMem_.enqueue(eff);
-		return false;
+		break;
 	default:
 		storeDirectRegisterSetEffectToQueue(SoundSource::FM, ch, eff);
-		return false;
+		break;
 	}
 }
 
@@ -802,14 +833,9 @@ void PlaybackManager::executeStoredEffectsFM(int ch)
 				tposeDlyValueFM_.at(uch) = ((eff.value & 0x80) ? -1 : 1) * (eff.value & 0x0f);
 				break;
 			case EffectType::VolumeDelay:
-			{
-				int count = eff.value >> 8;
-				if (count > 0) {
-					volDlyCntFM_.at(uch) = count;
-					volDlyValueFM_.at(uch) = eff.value & 0x00ff;
-				}
+				volDlyCntFM_.at(uch) = eff.value >> 8;
+				volDlyValueFM_.at(uch) = eff.value & 0x00ff;
 				break;
-			}
 			case EffectType::FBControl:
 				if (-1 < eff.value && eff.value < 8) opnaCtrl_->setFBControlFM(ch, eff.value);
 				break;
@@ -853,6 +879,25 @@ void PlaybackManager::executeStoredEffectsFM(int ch)
 				if (0 < eff.value) opnaCtrl_->setBrightnessFM(ch, eff.value - 0x80);
 				break;
 			}
+			case EffectType::EnvelopeReset:
+				envRstDlyCntFM_.at(uch) = eff.value;
+				break;
+			case EffectType::Retrigger:
+			{
+				int cnt = eff.value & 0x0f;
+				if (cnt) {
+					rtrgCntFM_.at(uch) = 0;
+					rtrgCntValueFM_.at(uch) = cnt;
+					rtrgVolValueFM_.at(uch) = ((eff.value & 0x80) ? 1 : -1) * ((eff.value & 0x70) >> 4);	// Reverse
+				}
+				break;
+			}
+			case EffectType::XVolumeSlide:
+			{
+				int factor = (eff.value >> 4) - (eff.value & 0x0f);
+				opnaCtrl_->setXVolumeSlideFM(ch, factor);
+				break;
+			}
 			default:
 				break;
 			}
@@ -863,7 +908,7 @@ void PlaybackManager::executeStoredEffectsFM(int ch)
 	}
 }
 
-bool PlaybackManager::storeEffectToMapSSG(int ch, const Effect& eff)
+void PlaybackManager::storeEffectToMapSSG(int ch, const Effect& eff)
 {
 	switch (eff.type) {
 	case EffectType::Arpeggio:
@@ -885,26 +930,25 @@ bool PlaybackManager::storeEffectToMapSSG(int ch, const Effect& eff)
 	case EffectType::HardEnvHighPeriod:
 	case EffectType::HardEnvLowPeriod:
 	case EffectType::AutoEnvelope:
+	case EffectType::Retrigger:
+	case EffectType::XVolumeSlide:
 		effOnKeyOnMem_[SoundSource::SSG].at(static_cast<size_t>(ch)).enqueue(eff);
-		return false;
+		break;
 	case EffectType::SpeedTempoChange:
 	case EffectType::Groove:
 		playbackSpeedEffMem_.enqueue(eff);
-		return false;
+		break;
 	case EffectType::NoteDelay:
-		if (eff.value < tickCounter_.lock()->getSpeed()) {
-			effOnStepBeginMem_[SoundSource::SSG].at(static_cast<size_t>(ch)).enqueue(eff);
-			return true;
-		}
-		return false;
+		effOnStepBeginMem_[SoundSource::SSG].at(static_cast<size_t>(ch)).enqueue(eff);
+		break;
 	case EffectType::PositionJump:
 	case EffectType::SongEnd:
 	case EffectType::PatternBreak:
 		posChangeEffMem_.enqueue(eff);
-		return false;
+		break;
 	default:
 		storeDirectRegisterSetEffectToQueue(SoundSource::SSG, ch, eff);
-		return false;
+		break;
 	}
 }
 
@@ -990,17 +1034,28 @@ void PlaybackManager::executeStoredEffectsSSG(int ch)
 				opnaCtrl_->setHardEnvelopePeriod(ch, false, eff.value);
 				break;
 			case EffectType::VolumeDelay:
-			{
-				int count = eff.value >> 8;
-				if (count > 0) {
-					volDlyCntSSG_.at(uch) = count;
-					volDlyValueSSG_.at(uch) = eff.value & 0x00ff;
-				}
+				volDlyCntSSG_.at(uch) = eff.value >> 8;
+				volDlyValueSSG_.at(uch) = eff.value & 0x00ff;
 				break;
-			}
 			case EffectType::AutoEnvelope:
 				opnaCtrl_->setAutoEnvelopeSSG(ch, (eff.value >> 4) - 8, eff.value & 0x0f);
 				break;
+			case EffectType::Retrigger:
+			{
+				int cnt = eff.value & 0x0f;
+				if (cnt) {
+					rtrgCntSSG_.at(uch) = 0;
+					rtrgCntValueSSG_.at(uch) = cnt;
+					rtrgVolValueSSG_.at(uch) = ((eff.value & 0x80) ? -1 : 1) * ((eff.value & 0x70) >> 4);
+				}
+				break;
+			}
+			case EffectType::XVolumeSlide:
+			{
+				int factor = (eff.value >> 4) - (eff.value & 0x0f);
+				opnaCtrl_->setXVolumeSlideSSG(ch, factor);
+				break;
+			}
 			default:
 				break;
 			}
@@ -1011,33 +1066,31 @@ void PlaybackManager::executeStoredEffectsSSG(int ch)
 	}
 }
 
-bool PlaybackManager::storeEffectToMapRhythm(int ch, const Effect& eff)
+void PlaybackManager::storeEffectToMapRhythm(int ch, const Effect& eff)
 {
 	switch (eff.type) {
 	case EffectType::Pan:
 	case EffectType::NoteCut:
 	case EffectType::MasterVolume:
 	case EffectType::VolumeDelay:
+	case EffectType::Retrigger:
 		effOnKeyOnMem_[SoundSource::RHYTHM].at(static_cast<size_t>(ch)).enqueue(eff);
-		return false;
+		break;
 	case EffectType::SpeedTempoChange:
 	case EffectType::Groove:
 		playbackSpeedEffMem_.enqueue(eff);
-		return false;
+		break;
 	case EffectType::NoteDelay:
-		if (eff.value < tickCounter_.lock()->getSpeed()) {
-			effOnStepBeginMem_[SoundSource::RHYTHM].at(static_cast<size_t>(ch)).enqueue(eff);
-			return true;
-		}
-		return false;
+		effOnStepBeginMem_[SoundSource::RHYTHM].at(static_cast<size_t>(ch)).enqueue(eff);
+		break;
 	case EffectType::PositionJump:
 	case EffectType::SongEnd:
 	case EffectType::PatternBreak:
 		posChangeEffMem_.enqueue(eff);
-		return false;
+		break;
 	default:
 		storeDirectRegisterSetEffectToQueue(SoundSource::RHYTHM, ch, eff);
-		return false;
+		break;
 	}
 }
 
@@ -1083,6 +1136,16 @@ void PlaybackManager::executeStoredEffectsRhythm(int ch)
 				}
 				break;
 			}
+			case EffectType::Retrigger:
+			{
+				int cnt = eff.value & 0x0f;
+				if (cnt) {
+					rtrgCntRhythm_.at(uch) = 0;
+					rtrgCntValueRhythm_.at(uch) = cnt;
+					rtrgVolValueRhythm_.at(uch) = ((eff.value & 0x80) ? -1 : 1) * ((eff.value & 0x70) >> 4);
+				}
+				break;
+			}
 			default:
 				break;
 			}
@@ -1093,7 +1156,7 @@ void PlaybackManager::executeStoredEffectsRhythm(int ch)
 	}
 }
 
-bool PlaybackManager::storeEffectToMapADPCM(int ch, const Effect& eff)
+void PlaybackManager::storeEffectToMapADPCM(int ch, const Effect& eff)
 {
 	switch (eff.type) {
 	case EffectType::Arpeggio:
@@ -1111,26 +1174,25 @@ bool PlaybackManager::storeEffectToMapADPCM(int ch, const Effect& eff)
 	case EffectType::NoteCut:
 	case EffectType::TransposeDelay:
 	case EffectType::VolumeDelay:
+	case EffectType::Retrigger:
+	case EffectType::XVolumeSlide:
 		effOnKeyOnMem_[SoundSource::ADPCM].front().enqueue(eff);
-		return false;
+		break;
 	case EffectType::SpeedTempoChange:
 	case EffectType::Groove:
 		playbackSpeedEffMem_.enqueue(eff);
-		return false;
+		break;
 	case EffectType::NoteDelay:
-		if (eff.value < tickCounter_.lock()->getSpeed()) {
-			effOnStepBeginMem_[SoundSource::ADPCM].front().enqueue(eff);
-			return true;
-		}
-		return false;
+		effOnStepBeginMem_[SoundSource::ADPCM].front().enqueue(eff);
+		break;
 	case EffectType::PositionJump:
 	case EffectType::SongEnd:
 	case EffectType::PatternBreak:
 		posChangeEffMem_.enqueue(eff);
-		return false;
+		break;
 	default:
 		storeDirectRegisterSetEffectToQueue(SoundSource::ADPCM, ch, eff);
-		return false;
+		break;
 	}
 }
 
@@ -1203,17 +1265,28 @@ void PlaybackManager::executeStoredEffectsADPCM()
 				break;
 			case EffectType::TransposeDelay:
 				tposeDlyCntADPCM_ = (eff.value & 0x70) >> 4;
-				tposeDlyValueADPCM_ = ((eff.value & 0x80) ? -1 : 1) * (eff.value & 0x0f);
+				tposeDlyValueADPCM_ = ((eff.value & 0x80) ? -1 : 1) * ((eff.value & 0x70) >> 4);
 				break;
 			case EffectType::VolumeDelay:
+				volDlyCntADPCM_ = eff.value >> 8;
+				volDlyValueADPCM_ = eff.value & 0x00ff;
+				break;
+			case EffectType::Retrigger:
 			{
-				int count = eff.value >> 8;
-				if (count > 0) {
-					volDlyCntADPCM_ = count;
-					volDlyValueADPCM_ = eff.value & 0x00ff;
+				int cnt = eff.value & 0x0f;
+				if (cnt) {
+					rtrgCntADPCM_ = 0;
+					rtrgCntValueADPCM_ = cnt;
+					rtrgVolValueADPCM_ = ((eff.value & 0x80) ? -1 : 1) * (eff.value & 0x0f);
 				}
 				break;
 			}
+			case EffectType::XVolumeSlide:
+			{
+				int factor = (eff.value >> 4) - (eff.value & 0x0f);
+				opnaCtrl_->setXVolumeSlideADPCM(factor);
+				break;
+			}
 			default:
 				break;
 			}
@@ -1297,8 +1370,11 @@ void PlaybackManager::effSpeedChange(int speed)
 
 void PlaybackManager::effTempoChange(int tempo)
 {
-	tickCounter_.lock()->setTempo(tempo);
-	tickCounter_.lock()->setGrooveState(GrooveState::Invalid);
+	auto tc = tickCounter_.lock();
+	if (tc->getGrooveEnabled() || tc->getTempo() != tempo) {
+		tc->setTempo(tempo);
+		tc->setGrooveState(GrooveState::Invalid);
+	}
 }
 
 void PlaybackManager::effGrooveChange(int num)
@@ -1318,34 +1394,39 @@ void PlaybackManager::tickProcess(int rest)
 		auto& curStep = song.getTrack(attrib.number)
 						.getPatternFromOrderNumber(playingPos_.order).getStep(playingPos_.step);
 		int ch = attrib.channelInSource;
+		bool hasDoneNoteDelay = false;
 		switch (attrib.source) {
-		case SoundSource::FM:		checkFMDelayEventsInTick(curStep, ch);		break;
-		case SoundSource::SSG:		checkSSGDelayEventsInTick(curStep, ch);		break;
-		case SoundSource::RHYTHM:	checkRhythmDelayEventsInTick(curStep, ch);	break;
-		case SoundSource::ADPCM:	checkADPCMDelayEventsInTick(curStep);		break;
+		case SoundSource::FM:		hasDoneNoteDelay = checkFMDelayEventsInTick(curStep, ch);		break;
+		case SoundSource::SSG:		hasDoneNoteDelay = checkSSGDelayEventsInTick(curStep, ch);		break;
+		case SoundSource::RHYTHM:	hasDoneNoteDelay = checkRhythmDelayEventsInTick(curStep, ch);	break;
+		case SoundSource::ADPCM:	hasDoneNoteDelay = checkADPCMDelayEventsInTick(curStep);		break;
 		}
 
 		if (rest == 1 && nextReadPos_.isValid() && attrib.source == SoundSource::FM && !isPlayingStep()) {
 			// Channel envelope reset before next key on
 			auto& step = song.getTrack(attrib.number)
 						 .getPatternFromOrderNumber(nextReadPos_.order).getStep(nextReadPos_.step);
+			bool hasNoteDelay = false;
 			for (int i = 0; i < Step::N_EFFECT; ++i) {
 				auto&& eff = effect_utils::validateEffect(attrib.source, step.getEffect(i));
 				if (eff.type == EffectType::NoteDelay && eff.value > 0) {	// Note delay check
 					opnaCtrl_->tickEvent(attrib.source, ch);
-					return;
+					hasNoteDelay = true;
+					break;
 				}
 			}
-			envelopeResetEffectFM(step, ch);
+			// Skip the statement below if envelope reset effect has cexecuted
+			if (!hasNoteDelay && envRstDlyCntFM_.at(static_cast<size_t>(ch))) envelopeResetEffectFM(step, ch);
 		}
-		else {
+		else if (!hasDoneNoteDelay) {
+			// Skip tick process when step process was called by note delay event.
 			opnaCtrl_->tickEvent(attrib.source, ch);
 		}
 	}
 	opnaCtrl_->updateRegisterStates();
 }
 
-void PlaybackManager::checkFMDelayEventsInTick(const Step& step, int ch)
+bool PlaybackManager::checkFMDelayEventsInTick(const Step& step, int ch)
 {
 	size_t uch = static_cast<size_t>(ch);
 	// Check volume delay
@@ -1357,20 +1438,31 @@ void PlaybackManager::checkFMDelayEventsInTick(const Step& step, int ch)
 	// Check transpose delay
 	if (!tposeDlyCntFM_.at(uch))
 		opnaCtrl_->setTransposeEffectFM(ch, tposeDlyValueFM_.at(uch));
+	// Check envelope reset delay
+	if (!envRstDlyCntFM_.at(uch))
+		opnaCtrl_->resetFMChannelEnvelope(ch);
+	// Check retrigger
+	if (!rtrgCntFM_.at(uch))
+		opnaCtrl_->retriggerKeyOnFM(ch, rtrgVolValueFM_.at(uch));
 	// Check note delay and envelope reset
-	checkFMNoteDelayAndEnvelopeReset(step, ch);
+	return checkFMNoteDelayAndEnvelopeReset(step, ch);
 }
 
-void PlaybackManager::checkFMNoteDelayAndEnvelopeReset(const Step& step, int ch)
+bool PlaybackManager::checkFMNoteDelayAndEnvelopeReset(const Step& step, int ch)
 {
 	int cnt = ntDlyCntFM_.at(static_cast<size_t>(ch));
 	if (!cnt) {
 		executeFMStepEvents(step, ch, true);
+		return true;
 	}
-	else if (cnt == 1) {
+
+	// Skip the statement below if envelope reset effect has cexecuted
+	if (cnt == 1 && envRstDlyCntFM_.at(static_cast<size_t>(ch))) {
 		// Channel envelope reset before next key on
 		envelopeResetEffectFM(step, ch);
 	}
+
+	return false;
 }
 
 void PlaybackManager::envelopeResetEffectFM(const Step& step, int ch)
@@ -1393,7 +1485,7 @@ void PlaybackManager::envelopeResetEffectFM(const Step& step, int ch)
 	}
 }
 
-void PlaybackManager::checkSSGDelayEventsInTick(const Step& step, int ch)
+bool PlaybackManager::checkSSGDelayEventsInTick(const Step& step, int ch)
 {
 	size_t uch = static_cast<size_t>(ch);
 	// Check volume delay
@@ -1405,12 +1497,18 @@ void PlaybackManager::checkSSGDelayEventsInTick(const Step& step, int ch)
 	// Check transpose delay
 	if (!tposeDlyCntSSG_.at(uch))
 		opnaCtrl_->setTransposeEffectSSG(ch, tposeDlyValueSSG_.at(uch));
+	// Check retrigger
+	if (!rtrgCntSSG_.at(uch))
+		opnaCtrl_->retriggerKeyOnSSG(ch, rtrgVolValueSSG_.at(uch));
 	// Check note delay
-	if (!ntDlyCntSSG_.at(uch))
+	if (!ntDlyCntSSG_.at(uch)) {
 		executeSSGStepEvents(step, ch, true);
+		return true;
+	}
+	return false;
 }
 
-void PlaybackManager::checkRhythmDelayEventsInTick(const Step& step, int ch)
+bool PlaybackManager::checkRhythmDelayEventsInTick(const Step& step, int ch)
 {
 	size_t uch = static_cast<size_t>(ch);
 	// Check volume delay
@@ -1419,12 +1517,18 @@ void PlaybackManager::checkRhythmDelayEventsInTick(const Step& step, int ch)
 	// Check note cut
 	if (!ntCutDlyCntRhythm_.at(uch))
 		opnaCtrl_->setKeyOnFlagRhythm(ch);
+	// Check retrigger
+	if (!rtrgCntRhythm_.at(uch))
+		opnaCtrl_->retriggerKeyOnFlagRhythm(ch, rtrgVolValueRhythm_.at(uch));
 	// Check note delay
-	if (!ntDlyCntRhythm_.at(uch))
+	if (!ntDlyCntRhythm_.at(uch)) {
 		executeRhythmStepEvents(step, ch, true);
+		return true;
+	}
+	return false;
 }
 
-void PlaybackManager::checkADPCMDelayEventsInTick(const Step& step)
+bool PlaybackManager::checkADPCMDelayEventsInTick(const Step& step)
 {
 	// Check volume delay
 	if (!volDlyCntADPCM_)
@@ -1435,9 +1539,15 @@ void PlaybackManager::checkADPCMDelayEventsInTick(const Step& step)
 	// Check transpose delay
 	if (!tposeDlyCntADPCM_)
 		opnaCtrl_->setTransposeEffectADPCM(tposeDlyValueADPCM_);
+	// Check retrigger
+	if (!rtrgCntADPCM_)
+		opnaCtrl_->retriggerKeyOnADPCM(rtrgVolValueADPCM_);
 	// Check note delay
-	if (!ntDlyCntADPCM_)
+	if (!ntDlyCntADPCM_) {
 		executeADPCMStepEvents(step, true);
+		return true;
+	}
+	return false;
 }
 
 void PlaybackManager::clearEffectMaps()
@@ -1456,17 +1566,27 @@ void PlaybackManager::clearEffectMaps()
 	}
 }
 
-void PlaybackManager::clearNoteDelayCounts()
+void PlaybackManager::clearDelayWithinStepCounts()
 {
 	std::fill(ntDlyCntFM_.begin(), ntDlyCntFM_.end(), -1);
 	std::fill(ntDlyCntSSG_.begin(), ntDlyCntSSG_.end(), -1);
 	std::fill(ntDlyCntRhythm_.begin(), ntDlyCntRhythm_.end(), -1);
 	ntDlyCntADPCM_ = -1;
+
+	std::fill(rtrgCntFM_.begin(), rtrgCntFM_.end(), -1);
+	std::fill(rtrgCntValueFM_.begin(), rtrgCntValueFM_.end(), -1);
+	std::fill(rtrgCntSSG_.begin(), rtrgCntSSG_.end(), -1);
+	std::fill(rtrgCntValueSSG_.begin(), rtrgCntValueSSG_.end(), -1);
+	std::fill(rtrgCntRhythm_.begin(), rtrgCntRhythm_.end(), -1);
+	std::fill(rtrgCntValueRhythm_.begin(), rtrgCntValueRhythm_.end(), -1);
+	rtrgCntADPCM_ = -1;
+	rtrgCntValueADPCM_ = -1;
 }
 
 void PlaybackManager::clearDelayBeyondStepCounts()
 {
 	std::fill(ntCutDlyCntFM_.begin(), ntCutDlyCntFM_.end(), -1);
+	std::fill(envRstDlyCntFM_.begin(), envRstDlyCntFM_.end(), -1);
 	std::fill(volDlyCntFM_.begin(), volDlyCntFM_.end(), -1);
 	std::fill(volDlyValueFM_.begin(), volDlyValueFM_.end(), -1);
 	std::fill(tposeDlyCntFM_.begin(), tposeDlyCntFM_.end(), -1);
@@ -1493,6 +1613,7 @@ void PlaybackManager::clearFMDelayBeyondStepCounts(int ch)
 {
 	size_t uch = static_cast<size_t>(ch);
 	ntCutDlyCntFM_.at(uch) = -1;
+	envRstDlyCntFM_.at(uch) = -1;
 	volDlyCntFM_.at(uch) = -1;
 	volDlyValueFM_.at(uch) = -1;
 	tposeDlyCntFM_.at(uch) = -1;
@@ -1528,22 +1649,34 @@ void PlaybackManager::clearADPCMDelayBeyondStepCounts()
 
 void PlaybackManager::updateDelayEventCounts()
 {
-	auto f = [](int x) { return (x == -1) ? x : --x; };
-	std::transform(ntDlyCntFM_.begin(), ntDlyCntFM_.end(), ntDlyCntFM_.begin(), f);
-	std::transform(ntDlyCntSSG_.begin(), ntDlyCntSSG_.end(), ntDlyCntSSG_.begin(), f);
-	std::transform(ntDlyCntRhythm_.begin(), ntDlyCntRhythm_.end(), ntDlyCntRhythm_.begin(), f);
+	static auto cd1 = [](int x) { return (x == -1) ? -1 : --x; };
+	std::transform(ntDlyCntFM_.begin(), ntDlyCntFM_.end(), ntDlyCntFM_.begin(), cd1);
+	std::transform(ntDlyCntSSG_.begin(), ntDlyCntSSG_.end(), ntDlyCntSSG_.begin(), cd1);
+	std::transform(ntDlyCntRhythm_.begin(), ntDlyCntRhythm_.end(), ntDlyCntRhythm_.begin(), cd1);
 	--ntDlyCntADPCM_;
-	std::transform(ntCutDlyCntFM_.begin(), ntCutDlyCntFM_.end(), ntCutDlyCntFM_.begin(), f);
-	std::transform(ntCutDlyCntSSG_.begin(), ntCutDlyCntSSG_.end(), ntCutDlyCntSSG_.begin(), f);
-	std::transform(ntCutDlyCntRhythm_.begin(), ntCutDlyCntRhythm_.end(), ntCutDlyCntRhythm_.begin(), f);
+	std::transform(ntCutDlyCntFM_.begin(), ntCutDlyCntFM_.end(), ntCutDlyCntFM_.begin(), cd1);
+	std::transform(ntCutDlyCntSSG_.begin(), ntCutDlyCntSSG_.end(), ntCutDlyCntSSG_.begin(), cd1);
+	std::transform(ntCutDlyCntRhythm_.begin(), ntCutDlyCntRhythm_.end(), ntCutDlyCntRhythm_.begin(), cd1);
 	--ntCutDlyCntADPCM_;
-	std::transform(volDlyCntFM_.begin(), volDlyCntFM_.end(), volDlyCntFM_.begin(), f);
-	std::transform(volDlyCntSSG_.begin(), volDlyCntSSG_.end(), volDlyCntSSG_.begin(), f);
-	std::transform(volDlyCntRhythm_.begin(), volDlyCntRhythm_.end(), volDlyCntRhythm_.begin(), f);
+	std::transform(volDlyCntFM_.begin(), volDlyCntFM_.end(), volDlyCntFM_.begin(), cd1);
+	std::transform(volDlyCntSSG_.begin(), volDlyCntSSG_.end(), volDlyCntSSG_.begin(), cd1);
+	std::transform(volDlyCntRhythm_.begin(), volDlyCntRhythm_.end(), volDlyCntRhythm_.begin(), cd1);
 	--volDlyCntADPCM_;
-	std::transform(tposeDlyCntFM_.begin(), tposeDlyCntFM_.end(), tposeDlyCntFM_.begin(), f);
-	std::transform(tposeDlyCntSSG_.begin(), tposeDlyCntSSG_.end(), tposeDlyCntSSG_.begin(), f);
+	std::transform(tposeDlyCntFM_.begin(), tposeDlyCntFM_.end(), tposeDlyCntFM_.begin(), cd1);
+	std::transform(tposeDlyCntSSG_.begin(), tposeDlyCntSSG_.end(), tposeDlyCntSSG_.begin(), cd1);
 	--tposeDlyCntADPCM_;
+	std::transform(envRstDlyCntFM_.begin(), envRstDlyCntFM_.end(), envRstDlyCntFM_.begin(), cd1);
+
+	static auto cd2 = [](int& cnt, int max) {
+		if (cnt != -1) {
+			if (cnt) --cnt;
+			else cnt = max - 1;
+		}
+	};
+	for (size_t uch = 0; uch < rtrgCntFM_.size(); ++uch) cd2(rtrgCntFM_.at(uch), rtrgCntValueFM_.at(uch));
+	for (size_t uch = 0; uch < rtrgCntSSG_.size(); ++uch) cd2(rtrgCntSSG_.at(uch), rtrgCntValueSSG_.at(uch));
+	for (size_t uch = 0; uch < rtrgCntRhythm_.size(); ++uch) cd2(rtrgCntRhythm_.at(uch), rtrgCntValueRhythm_.at(uch));
+	cd2(rtrgCntADPCM_, rtrgCntValueADPCM_);
 }
 
 void PlaybackManager::checkPlayPosition(int maxStepSize)
@@ -1569,14 +1702,14 @@ void PlaybackManager::retrieveChannelStates()
 	std::vector<bool> isSetPanFM(fmch, false), isSetVolSldFM(fmch, false), isSetDtnFM(fmch, false);
 	std::vector<bool> isSetFBCtrlFM(fmch, false), isSetTLCtrlFM(fmch, false), isSetMLCtrlFM(fmch, false);
 	std::vector<bool> isSetARCtrlFM(fmch, false), isSetDRCtrlFM(fmch, false), isSetRRCtrlFM(fmch, false);
-	std::vector<bool> isSetBrightFM(fmch, false), isSetFiDtnFM(fmch, false);
+	std::vector<bool> isSetBrightFM(fmch, false), isSetFiDtnFM(fmch, false), isSetXVolSldFM(fmch, false);
 	std::vector<bool> isSetInstSSG(3, false), isSetVolSSG(3, false), isSetArpSSG(3, false), isSetPrtSSG(3, false);
 	std::vector<bool> isSetVibSSG(3, false), isSetTreSSG(3, false), isSetVolSldSSG(3, false), isSetDtnSSG(3, false);
-	std::vector<bool> isSetTNMixSSG(3, false), isSetFiDtnSSG(3, false);
+	std::vector<bool> isSetTNMixSSG(3, false), isSetFiDtnSSG(3, false), isSetXVolSldSSG(3, false);
 	std::vector<bool> isSetVolRhythm(6, false), isSetPanRhythm(6, false);
 	bool isSetInstADPCM(false), isSetVolADPCM(false), isSetArpADPCM(false), isSetPrtADPCM(false);
 	bool isSetVibADPCM(false), isSetTreADPCM(false), isSetPanADPCM(false), isSetVolSldADPCM(false);
-	bool isSetDtnADPCM(false), isSetFiDtnADPCM(false);
+	bool isSetDtnADPCM(false), isSetFiDtnADPCM(false), isSetXVolSldADPCM(false);
 	bool isSetMVolRhythm = false;
 	bool isSetNoisePitchSSG = false;
 	bool isSetHardEnvPeriodHighSSG = false;
@@ -1776,6 +1909,15 @@ void PlaybackManager::retrieveChannelStates()
 							}
 						}
 						break;
+					case EffectType::XVolumeSlide:
+						if (!isSetXVolSldFM[uch]) {
+							isSetXVolSldFM[uch] = true;
+							if (isPrevPos) {
+								int factor = (eff.value >> 4) - (eff.value & 0x0f);
+								opnaCtrl_->setXVolumeSlideFM(ch, factor);
+							}
+						}
+						break;
 					default:
 						break;
 					}
@@ -1913,6 +2055,15 @@ void PlaybackManager::retrieveChannelStates()
 							if (isPrevPos) opnaCtrl_->setAutoEnvelopeSSG(ch, (eff.value >> 4) - 8, eff.value & 0x0f);
 						}
 						break;
+					case EffectType::XVolumeSlide:
+						if (!isSetXVolSldSSG[uch]) {
+							isSetXVolSldSSG[uch] = true;
+							if (isPrevPos) {
+								int factor = (eff.value >> 4) - (eff.value & 0x0f);
+								opnaCtrl_->setXVolumeSlideSSG(ch, factor);
+							}
+						}
+						break;
 					default:
 						break;
 					}
@@ -2047,11 +2198,13 @@ void PlaybackManager::retrieveChannelStates()
 						break;
 					case EffectType::SpeedTempoChange:
 						if (!(speedStates & 0x4)) {
-							if (eff.value < 0x20 && !(speedStates & 0x1)) {	// Speed change
-								speedStates |= 0x1;
-								if (isPrevPos) effSpeedChange(eff.value);
+							if (eff.value < 0x20) {				// Speed change
+								if (!(speedStates & 0x1)) {
+									speedStates |= 0x1;
+									if (isPrevPos) effSpeedChange(eff.value);
+								}
 							}
-							else if (!(speedStates & 0x2)) {			// Tempo change
+							else if (!(speedStates & 0x2)) {	// Tempo change
 								speedStates |= 0x2;
 								if (isPrevPos) effTempoChange(eff.value);
 							}
@@ -2075,6 +2228,15 @@ void PlaybackManager::retrieveChannelStates()
 							if (isPrevPos) opnaCtrl_->setFineDetuneADPCM(eff.value - 0x80);
 						}
 						break;
+					case EffectType::XVolumeSlide:
+						if (!isSetXVolSldADPCM) {
+							isSetXVolSldADPCM = true;
+							if (isPrevPos) {
+								int factor = (eff.value >> 4) - (eff.value & 0x0f);
+								opnaCtrl_->setXVolumeSlideADPCM(factor);
+							}
+						}
+						break;
 					default:
 						break;
 					}
