@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2019-2021 Rerrah
+ * Copyright (C) 2019-2022 Rerrah
  *
  * Permission is hereby granted, free of charge, to any person
  * obtaining a copy of this software and associated documentation
@@ -59,6 +59,9 @@ public:
 	const_iterator end() const noexcept { return mem_.end(); }
 	const_iterator cend() const noexcept { return mem_.cend(); }
 
+	iterator erase(iterator position) { return mem_.erase(position); }
+	iterator erase(const_iterator position) { return mem_.erase(position); }
+
 private:
 	std::vector<Effect> mem_;
 };
@@ -131,7 +134,6 @@ private:
 	void checkValidPosition();
 
 	void stepProcess();
-	std::unordered_map<SoundSource, std::vector<bool>> isNoteDelay_;
 
 	void executeFMStepEvents(const Step& step, int ch, bool calledByNoteDelay = false);
 	void executeSSGStepEvents(const Step& step, int ch, bool calledByNoteDelay = false);
@@ -151,13 +153,13 @@ private:
 	std::unordered_map<SoundSource, DirectRegisterSetSource> directRegisterSets_;
 
 	bool executeStoredEffectsGlobal();
-	bool storeEffectToMapFM(int ch, const Effect& eff);
+	void storeEffectToMapFM(int ch, const Effect& eff);
 	void executeStoredEffectsFM(int ch);
-	bool storeEffectToMapSSG(int ch, const Effect& eff);
+	void storeEffectToMapSSG(int ch, const Effect& eff);
 	void executeStoredEffectsSSG(int ch);
-	bool storeEffectToMapRhythm(int ch, const Effect& eff);
+	void storeEffectToMapRhythm(int ch, const Effect& eff);
 	void executeStoredEffectsRhythm(int ch);
-	bool storeEffectToMapADPCM(int ch, const Effect& eff);
+	void storeEffectToMapADPCM(int ch, const Effect& eff);
 	void executeStoredEffectsADPCM();
 	void storeDirectRegisterSetEffectToQueue(SoundSource src, int ch, const Effect& eff);
 	void executeDirectRegisterSetEffect(DirectRegisterSetQueue& queue);
@@ -171,26 +173,59 @@ private:
 
 	void tickProcess(int rest);
 
-	void checkFMDelayEventsInTick(const Step& step, int ch);
-	void checkFMNoteDelayAndEnvelopeReset(const Step& step, int ch);
+	/**
+	 * @brief checkFMDelayEventsInTick
+	 * @param step
+	 * @param ch
+	 * @return \c true \c if note delay is occurred.
+	 */
+	bool checkFMDelayEventsInTick(const Step& step, int ch);
+	/**
+	 * @brief checkFMNoteDelayAndEnvelopeReset
+	 * @param step
+	 * @param ch
+	 * @return \c true \c if note delay is occurred.
+	 */
+	bool checkFMNoteDelayAndEnvelopeReset(const Step& step, int ch);
 	void envelopeResetEffectFM(const Step& step, int ch);
-	void checkSSGDelayEventsInTick(const Step& step, int ch);
-	void checkRhythmDelayEventsInTick(const Step& step, int ch);
-	void checkADPCMDelayEventsInTick(const Step& step);
+	/**
+	 * @brief checkSSGDelayEventsInTick
+	 * @param step
+	 * @param ch
+	 * @return \c true \c if note delay is occurred.
+	 */
+	bool checkSSGDelayEventsInTick(const Step& step, int ch);
+	/**
+	 * @brief checkRhythmDelayEventsInTick
+	 * @param step
+	 * @param ch
+	 * @return \c true \c if note delay is occurred.
+	 */
+	bool checkRhythmDelayEventsInTick(const Step& step, int ch);
+	/**
+	 * @brief checkADPCMDelayEventsInTick
+	 * @param step
+	 * @return \c true \c if note delay is occurred.
+	 */
+	bool checkADPCMDelayEventsInTick(const Step& step);
 
-	std::vector<int> ntDlyCntFM_, ntCutDlyCntFM_, volDlyCntFM_;
-	std::vector<int> ntDlyCntSSG_, ntCutDlyCntSSG_, volDlyCntSSG_;
-	std::vector<int> ntDlyCntRhythm_, ntCutDlyCntRhythm_, volDlyCntRhythm_;
-	int ntDlyCntADPCM_, ntCutDlyCntADPCM_, volDlyCntADPCM_;
+	std::vector<int> ntDlyCntFM_, ntCutDlyCntFM_, volDlyCntFM_, envRstDlyCntFM_, rtrgCntFM_;
+	std::vector<int> ntDlyCntSSG_, ntCutDlyCntSSG_, volDlyCntSSG_, rtrgCntSSG_;
+	std::vector<int> ntDlyCntRhythm_, ntCutDlyCntRhythm_, volDlyCntRhythm_, rtrgCntRhythm_;
+	int ntDlyCntADPCM_, ntCutDlyCntADPCM_, volDlyCntADPCM_, rtrgCntADPCM_;
 	std::vector<int> volDlyValueFM_, volDlyValueSSG_, volDlyValueRhythm_;
 	int volDlyValueADPCM_;
 	std::vector<int> tposeDlyCntFM_, tposeDlyCntSSG_;
 	int tposeDlyCntADPCM_;
 	std::vector<int> tposeDlyValueFM_, tposeDlyValueSSG_;
 	int tposeDlyValueADPCM_;
+	std::vector<int> rtrgCntValueFM_, rtrgCntValueSSG_, rtrgCntValueRhythm_;
+	int rtrgCntValueADPCM_;
+	std::vector<int> rtrgVolValueFM_, rtrgVolValueSSG_, rtrgVolValueRhythm_;
+	int rtrgVolValueADPCM_;
 
 	void clearEffectMaps();
-	void clearNoteDelayCounts();
+	void clearDelayWithinStepCounts();
 	void clearDelayBeyondStepCounts();
 	void clearFMDelayBeyondStepCounts(int ch);
 	void clearSSGDelayBeyondStepCounts(int ch);
