@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2018-2021 Rerrah
+ * Copyright (C) 2018-2022 Rerrah
  *
  * Permission is hereby granted, free of charge, to any person
  * obtaining a copy of this software and associated documentation
@@ -29,52 +29,56 @@
 #include <string>
 #include <stdexcept>
 
+// ***********
+// The use of constexpr function was avoided because old `g++ -std=c++1y` does not fully support C++14.
+// ***********
+
 class Version
 {
 public:
-	static uint32_t ofApplicationInBCD();
+	static /* constexpr */ uint32_t ofApplicationInBCD();
 	static std::string ofApplicationInString();
 
-	static uint32_t ofModuleFileInBCD();
+	static /* constexpr */ uint32_t ofModuleFileInBCD();
 	static std::string ofModuleFileInString();
 
-	static uint32_t ofInstrumentFileInBCD();
+	static /* constexpr */ uint32_t ofInstrumentFileInBCD();
 	static std::string ofInstrumentFileInString();
 
-	static uint32_t ofBankFileInBCD();
+	static /* constexpr */ uint32_t ofBankFileInBCD();
 	static std::string ofBankFileInString();
 
-	static uint32_t toBCD(unsigned int major, unsigned int minor, unsigned int revision);
+	static /* constexpr */ uint32_t toBCD(unsigned int major, unsigned int minor, unsigned int revision);
 	static std::string toString(unsigned int major, unsigned int minor, unsigned int revision);
 
 private:
 	// Application version
 	static constexpr unsigned int appMajor			= 0;
-	static constexpr unsigned int appMinor			= 4;
-	static constexpr unsigned int appRevision		= 6;
+	static constexpr unsigned int appMinor			= 5;
+	static constexpr unsigned int appRevision		= 1;
 
 	// Module file version
 	static constexpr unsigned int modFileMajor		= 1;
-	static constexpr unsigned int modFileMinor		= 5;
+	static constexpr unsigned int modFileMinor		= 6;
 	static constexpr unsigned int modFileRevision	= 0;
 
 	// Instrument file version
 	static constexpr unsigned int instFileMajor		= 1;
-	static constexpr unsigned int instFileMinor		= 4;
+	static constexpr unsigned int instFileMinor		= 5;
 	static constexpr unsigned int instFileRevision	= 0;
 
 	// Bank file version
 	static constexpr unsigned int bankFileMajor		= 1;
-	static constexpr unsigned int bankFileMinor		= 2;
+	static constexpr unsigned int bankFileMinor		= 3;
 	static constexpr unsigned int bankFileRevision	= 0;
 
-	Version() = default;
+	constexpr Version() = default;
 
-	static uint8_t uitobcd(const uint8_t v);
+	static /* constexpr */ uint8_t uitobcd(uint8_t v);
 };
 
 //===============================================
-inline uint32_t Version::ofApplicationInBCD()
+inline /* constexpr */ uint32_t Version::ofApplicationInBCD()
 {
 	return toBCD(appMajor, appMinor, appRevision);
 }
@@ -84,7 +88,7 @@ inline std::string Version::ofApplicationInString()
 	return toString(appMajor, appMinor, appRevision);
 }
 
-inline uint32_t Version::ofModuleFileInBCD()
+inline /* constexpr */ uint32_t Version::ofModuleFileInBCD()
 {
 	return toBCD(modFileMajor, modFileMinor, modFileRevision);
 }
@@ -94,7 +98,7 @@ inline std::string Version::ofModuleFileInString()
 	return toString(modFileMajor, modFileMinor, modFileRevision);
 }
 
-inline uint32_t Version::ofInstrumentFileInBCD()
+inline /* constexpr */ uint32_t Version::ofInstrumentFileInBCD()
 {
 	return toBCD(instFileMajor, instFileMinor, instFileRevision);
 }
@@ -104,7 +108,7 @@ inline std::string Version::ofInstrumentFileInString()
 	return toString(instFileMajor, instFileMinor, instFileRevision);
 }
 
-inline uint32_t Version::ofBankFileInBCD()
+inline /* constexpr */ uint32_t Version::ofBankFileInBCD()
 {
 	return toBCD(bankFileMajor, bankFileMinor, bankFileRevision);
 }
@@ -114,7 +118,7 @@ inline std::string Version::ofBankFileInString()
 	return toString(bankFileMajor, bankFileMinor, bankFileRevision);
 }
 
-inline uint32_t Version::toBCD(unsigned int major, unsigned int minor, unsigned int revision)
+inline /* constexpr */ uint32_t Version::toBCD(unsigned int major, unsigned int minor, unsigned int revision)
 {
 	uint32_t maj = uitobcd(static_cast<uint8_t>(major));
 	uint32_t min = uitobcd(static_cast<uint8_t>(minor));
@@ -127,11 +131,11 @@ inline std::string Version::toString(unsigned int major, unsigned int minor, uns
 	return std::to_string(major) + "." + std::to_string(minor) + "." + std::to_string(revision);
 }
 
-inline uint8_t Version::uitobcd(const uint8_t v)
+inline /* constexpr */ uint8_t Version::uitobcd(uint8_t v)
 {
-	if (v > 99) throw std::out_of_range("Out of range.");
+	 if (v > 99) throw std::out_of_range("[Version::uitobcd] Argument error: out of range");
 
-	uint8_t high = v / 10;
-	uint8_t low = v % 10;
+	uint8_t high = v / 10u;
+	uint8_t low = v % 10u;
 	return static_cast<uint8_t>(high << 4) + low;
 }
